% clear
clear variables;
close all;
clc;
%% physical constants
e = 1.60217662e-19;             %elementary charge (in C)
hbar = 6.62607015e-34/(2*pi);   %reduced Planck constant (in Js)
me = 9.109384e-31;              %electron rest mass (in kg)
c_light = 299792458;            %Speed of light in vacuum (in m/s)
kB = 1.380649e-23;              %Boltzmann constant (in J/K)
mu0 = 4*pi*1e-7;                %premeability of free space (in N/A^2)
e0 = 1/(mu0*c_light^2);         %permittivity of free space (in F/m)
%% setup figure
fs = 12; %fontsize
linewidth = 2;
set(groot, 'defaultAxesTickLabelInterpreter','tex');
set(groot, 'defaultLegendInterpreter','tex');
set(groot, 'defaultTextInterpreter','tex');
set(groot, 'defaultAxesFontSize',fs);
set(groot, 'defaultTextFontSize',fs);
set(groot, 'defaultAxesLabelFontSize', fs);
set(groot, 'defaultAxesLabelFontSizeMultiplier', 1);
set(groot, 'defaultLineLineWidth', linewidth);
set(groot, 'defaultAxesLineWidth', linewidth);

fig1_name = 'ResultOverview';
fig1 = figure('Name', fig1_name, 'Position', [10, 50, 400, 300]);

datapath_savefigure = '../EvaluationCode/Figures';
sample_id = 4; %set to 1 for Anthanthrene, perpendicular polarization and to 2 for TTF-BTD, parallel polarization,
                %and for the SI: 3 (Anthanthrene, par. pol), 4 (TTF-BTD,
                %per pol)

save_data = 0; %control parameter if Stark-signal are saved or note
mycolormap = diverging_map(linspace(0,1,100),[0,0,1],[1,0,0]);
mycolormap2 = diverging_map(linspace(0,1,100),[1,1,1],[0.6,0,0]);
%% load EO-signal
data_path = '../RawData/20220308/EO_signal_diffConfiguration/';
data11 = dlmread([data_path, 'time0205date090322_initCond_withQuartz.txt']);
inc = 75; %considered increments between scan positions
dt = inc*2*0.1e-6/c_light;
time_eo = (0:length(data11(:,2))-1)*dt*1e12;
signal_eo = data11(:,2) - mean(data11(:,2));
signal_eo = signal_eo/max(abs(signal_eo));
%% get data toluene
measurementday = '20220304';
if sample_id == 1
    dataname = '02_25_13 Toluene_per_transient.txt';
elseif sample_id == 2
    dataname = '01_24_01 Toluene_par_transient.txt';
elseif sample_id == 3
    dataname = '01_24_01 Toluene_par_transient.txt';
elseif sample_id == 4
    dataname = '02_25_13 Toluene_per_transient.txt';
end
chirp_correction_case = 1; %set to 1 for Cuvette and 2 for Capillary
time_avg_width = 0; 
rem_bg = 25; 
[wave_bg, deltaA_bg, time_cc_bg, TAS_data_2DM_bg] = get_deltaA_from_TASdata(measurementday, dataname, time_avg_width, rem_bg, chirp_correction_case);
%% movemean for background
N_movmean_time = 1;
N_movmean_spec = 300; 
B = ones(N_movmean_spec,N_movmean_time)/(N_movmean_time*N_movmean_spec);
TAS_data_2DM_bg_conv = conv2(TAS_data_2DM_bg,B,'same');
TAS_data_2DM_bg = TAS_data_2DM_bg_conv;
%% get data sample
if sample_id == 1
    measurementday = '20210908';
    dataname = '00_55_23 Sam1  Cuvette per_transient.txt';
elseif sample_id == 2
    measurementday = '20210909';
    dataname = '22_57_24 Sam2 par_transient.txt';
elseif sample_id == 3
    measurementday = '20210908';
    dataname = '23_00_53 Sam1  Cuvette par_transient.txt';
elseif sample_id == 4
    measurementday = '20210909';
    dataname = '01_20_56 Sam2 per_transient.txt';
end


chirp_correction_case = 1; %set to 1 for Cuvette and 2 for Capillary

titletext = strrep(erase(dataname,".txt"), '_', '\_');

time_avg_width = 0; %3
rem_bg = 10; %25;
[wave, deltaA, time_cc, TAS_data_2DM] = get_deltaA_from_TASdata(measurementday, dataname, time_avg_width, rem_bg, chirp_correction_case);

%% background substraction
if sample_id == 1
    time_shift_bg = -0.45; %in ps %for Sample 1, para
elseif sample_id == 2
    time_shift_bg = 0.12; %in ps %for Sample 2, para
elseif sample_id == 3
    time_shift_bg = 0.1; %in ps %for Sample 1, per
elseif sample_id == 4
    time_shift_bg = -0.5; %in ps %for Sample 2, perend
end
ind_bg = find(wave_bg > 400 & wave_bg < 600);
OD_mean_tol = mean(-log10(TAS_data_2DM_bg(ind_bg,:)+1)*1e6, 1);
ind_bg = find(wave > 400 & wave < 600);
OD_mean_sam = mean(-log10(TAS_data_2DM(ind_bg,:)+1)*1e6, 1);
OD_abs_mean_sam = mean(abs(log10(TAS_data_2DM(ind_bg,:)+1)*1e6), 1);
%% background correction
[Time_bg, Wave_bg] = meshgrid(time_cc_bg+time_shift_bg, wave_bg);
[Time, Wave] = meshgrid(time_cc, wave);
TAS_data_2DM_bg_interp = interp2(Time_bg, Wave_bg, TAS_data_2DM_bg, Time, Wave, 'spline', 0);

ind_bg = find(wave_bg > 450 & wave_bg < 600);;
TAS_data_2DM_cor = TAS_data_2DM - TAS_data_2DM_bg_interp;

%% get THz data from Stark-fitting
THz_datapath = '../RawData/StarkFit_DataSets/';
sample1_filename = 'Sample1_Cuvette/sam1_cuvette_test9';
sample2_filename = 'Sample2_Cuvette/sam2_cuvette_test8';

if sample_id == 1 || sample_id == 3
    alpha_pol = [90, 0]; %polarizations (in deg) (90deg is horizontal)
    alpha_inc = [90, 0]; %incidence angle (in deg)
    sample_filename = sample1_filename;
    titletext = 'Anthanthrene';
    legpos = 'northeast';
    xrange = [440, 550];
elseif sample_id == 2 || sample_id == 4
    alpha_pol = [90, 0]; %polarizations (in deg) (90deg is horizontal)
    alpha_inc = [90, 0]; %incidence angle (in deg)
    sample_filename = sample2_filename;
    titletext = 'TTF-BTD';
    legpos = 'northeast';
    xrange = [400, 650];
end

data_absfit = dlmread([THz_datapath, sample_filename, '_ABSFIT.txt'], '',1,0);
data_stkfit1 = dlmread([THz_datapath, sample_filename, '_STKFIT1.txt'], '',1,0);
data_stkfit2 = dlmread([THz_datapath, sample_filename, '_STKFIT2.txt'], '',1,0);

%get angle between probe and stark electric field
for counter = 1:2
    if alpha_pol(counter) == 90
        beta(counter) = 90-alpha_inc(counter);
    elseif alpha_pol(counter) == 0
        beta(counter) = 90;
    else
        disp('Can not handle this polarization angle. Please check.')
    end
end

if sample_id == 1
    data_absfit_sam1_thz = data_absfit;
    data_stkfit1_sam1_thz = data_stkfit1;
    data_stkfit2_sam1_thz = data_stkfit2;
    beta_sam1_thz = beta;
else
    data_absfit_sam2_thz = data_absfit;
    data_stkfit1_sam2_thz = data_stkfit1;
    data_stkfit2_sam2_thz = data_stkfit2;
    beta_sam2_thz = beta;
end

%% load molecular structure
structure_datapath = '../Schematics/MarvinSketch/';

if sample_id == 1 || sample_id == 3
    molecule_structure =  imread([structure_datapath 'Anthanthrene.tiff']);
    molecule_structure = imcomplement(molecule_structure(:,:,1:3));
elseif sample_id == 2 || sample_id == 4
    molecule_structure =  imread([structure_datapath 'TTF-BTD.tiff']);
    molecule_structure = imcomplement(molecule_structure(:,:,1:3));
end
%% generate figure
set(fig1, 'Position', [10 61 500 400]);
if sample_id == 1 || sample_id == 3
    if sample_id == 1
        fig1_name = 'Anthanthrene_ResultOverview_perpol';
    else
        fig1_name = 'Anthanthrene_ResultOverview_parpol';
    end
    wl_lim = [400, 550];
    col_lim = [-800, 800];
    signal_lim = [-90, 700];
    eo_scale = 450;
    eo_timeoffset = 0.02; %in ps
    ind_bg = find(wave > 410 & wave < 540);
    ind_wave = find(wave > 482 & wave < 512); 
    time_avg_width = 3;
    OD_abs_mean_offset = 0; %15;
    col_wave = [0.8, 0.4, 0.2];
    col_bg = [0.4, 0.2, 0.8];
    col_signal = [0.2, 0.8, 0.4];
    deltaA_ticks = [-800, 0, 800];
elseif sample_id == 2 || sample_id == 4
    if sample_id == 2
        fig1_name = 'TTF-BTD_ResultOverview_parpol';
        signal_lim_scale = 1; 
    else
        fig1_name = 'TTF-BTD_ResultOverview_perpol';
        signal_lim_scale = 0.7; %to accout for the smaller signal for this polarization
    end
    wl_lim = [370, 670];
    col_lim = [-150, 150]*signal_lim_scale;
    signal_lim = [-30, 200*signal_lim_scale];
    eo_scale = 110*signal_lim_scale;
    eo_timeoffset = 0;
    ind_bg = find(wave > 390 & wave < 650);
    ind_wave = find(wave > 500 & wave < 550); 
    time_avg_width = 3;
    OD_abs_mean_offset = 0; 
    col_wave = [0.8, 0.4, 0.2];
    col_bg = [0.4, 0.2, 0.8];
    col_signal = [0.2, 0.8, 0.4];
    deltaA_ticks = [-150, 150]*signal_lim_scale;
end
time_lim = [-1.5, 1];

figure(fig1); clf;
subplot(3,3,[1,2,4,5]);

%correct time and wavelength points for plotting with pcolor (such that
%this points are in the middle of each rectangle
dt = mean(diff(time_cc));
time_cc_plot = time_cc - dt/2; 
wave_plot = wave; 
wave_plot(1) = wave(1)-(wave(1)-wave(2));
for counter = 2:length(wave)
    wave_plot(counter) = wave(counter)-(wave(counter-1)-wave(counter));
end
hp = pcolor(time_cc_plot, wave_plot, -log10(TAS_data_2DM_cor+1)*1e6);
hp.EdgeColor = 'none';
set(gca, 'ydir', 'normal')
colormap(mycolormap);
hcol = colorbar;
hcol.Location = 'northoutside';
caxis(col_lim);
hcol.Ticks = [col_lim(1), 0, col_lim(2)]; 

ylabel('Wavelength (nm)');
set(get(hcol,'title'),'string','\Delta A (\muOD)');
xlim(time_lim);
ylim([wl_lim])


% spectra
dt = mean(diff(time_cc));
ind_time = find(time_cc > -time_avg_width*dt-10*eps & time_cc < time_avg_width*dt+10*eps); %find(wave > 400 & wave < 650); %find(wave > 450 & wave < 540); %find(wave > 470 & wave < 503);
OD_spec_mean = -log10(mean(TAS_data_2DM_cor(:, ind_time)+1, 2))*1e6;

%add rectangle box
t1 = time_cc(ind_time(1));
t2 = time_cc(ind_time(end));
wl1 = wl_lim(1);
wl2 = wl_lim(2);
h_box1 = rectangle('Position', [t1, wl1, t2-t1, wl2-wl1], 'EdgeColor',col_signal, 'Linewidth', 2, 'LineStyle', ':');

t1 = time_lim(1);
t2 = time_lim(2);
wl1 = wave(ind_wave(end));
wl2 = wave(ind_wave(1));
h_box2 = rectangle('Position', [t1, wl1, t2-t1, wl2-wl1], 'EdgeColor',col_wave, 'Linewidth', 2, 'LineStyle', ':');

t1 = time_lim(1);
t2 = time_lim(2);
wl1 = wave(ind_bg(end));
wl2 = wave(ind_bg(1));
h_box2 = rectangle('Position', [t1, wl1, t2-t1, wl2-wl1], 'EdgeColor',col_bg, 'Linewidth', 2, 'LineStyle', ':');

box on;
set(gca, 'Layer', 'top');

% time signals
OD_mean = mean(-log10(TAS_data_2DM_cor(ind_bg,:)+1)*1e6, 1);
OD_abs_mean = mean(abs(-log10(TAS_data_2DM_cor(ind_wave,:)+1))*1e6, 1);

subplot(3,3,[3, 6]);
plot(OD_spec_mean, wave, 'Color', col_signal);
grid on;
ylim(wl_lim)
xlim(col_lim);
xticks(deltaA_ticks);
hcol = colorbar;
hcol.Location = 'northoutside';
set(hcol, 'Visible', 'off');
xlabel('\Delta A (\muOD)');

%add stkfit data
data_stkfit = data_stkfit1;
wavenum = data_stkfit(:,1);
A_chi = data_stkfit(:,4);
B_chi = data_stkfit(:,5);
C_chi = data_stkfit(:,6);
wave_fit = 1./wavenum*1e9*1e-2;
hold on;
if sample_id == 1 || sample_id == 3
    h_bchi = plot(B_chi./max(abs(B_chi))*col_lim(2)*0.7, wave_fit, '--', 'Color', [1,1,1]*0.3);
    h_cchi = plot(C_chi./max(abs(C_chi))*col_lim(2)*0.7, wave_fit, ':', 'Color', [1,1,1]*0.3);
    hleg1 = legend([h_bchi, h_cchi], '1^{st} order deri.', '2^{nd} order deri.', 'Interpreter', 'tex');
elseif sample_id == 2 || sample_id == 4
    h_bchi = plot(B_chi./max(abs(B_chi))*col_lim(2)*0.7, wave_fit, '--', 'Color', [1,1,1]*0.3);
    h_cchi = plot(C_chi./max(abs(C_chi))*col_lim(2)*0.7, wave_fit, ':', 'Color', [1,1,1]*0.3);
    hleg1 = legend([h_bchi, h_cchi], '1^{st} order deri.', '2^{nd} order deri.', 'Interpreter', 'tex');
end
hleg1.Position = [ 0.6931    0.8475    0.2089    0.1125];
box on;
set(gca, 'Layer', 'top');

subplot(3,3,[7,8]); hold on;
h_ODmean = plot(time_cc, OD_mean, 'Color', col_bg);
h_ODabs = plot(time_cc, OD_abs_mean-OD_abs_mean_offset, 'Color', col_wave);
h_eo = plot(time_eo-3+eo_timeoffset, signal_eo.^2*eo_scale, '--', 'color', 'k');
ylabel('\Delta A (\muOD)');
grid on;
xlim(time_lim);
ylim(signal_lim);
xlabel('Time (ps)');

hleg2 = legend([h_ODmean, h_ODabs, h_eo], '$\overline{\Delta A}$', '$\overline{|\Delta A|}$',...
    '$E_{THz}^2$', 'Interpreter','Latex', 'Orientation','horizontal');
hleg2.Location = 'Northwest';
box on;
set(gca, 'Layer', 'top');

subplot(3,3,[9]); hold on;
image(molecule_structure);
set(gca, 'ydir', 'reverse');
set(gca, 'Visible', 'off');
axis image
drawnow();
pos = get(gca, 'Position');
set(gca, 'Position', pos+[-0.06, -0.08, 0.15, 0.05]);

set(findall(gcf,'-property','FontSize'),'FontSize',fs)

fs2 = 16;
if sample_id == 1 || sample_id == 3
    h1 = text(-3100, -2050, 'a', 'Fontsize', fs2, 'FontWeight','bold');
    h2 = text(116, -2050, 'b', 'Fontsize', fs2, 'FontWeight','bold');
    h3 = text(-3100, 400, 'c', 'Fontsize', fs2, 'FontWeight','bold');
    h4 = text(100, 50, 'd', 'Fontsize', fs2, 'FontWeight','bold');
else
    h1 = text(-1600, -1300, 'a', 'Fontsize', fs2, 'FontWeight','bold');
    h2 = text(300, -1300, 'b', 'Fontsize', fs2, 'FontWeight','bold');
    h3 = text(-1600, 150, 'c', 'Fontsize', fs2, 'FontWeight','bold');
    h4 = text(50, 0, 'd', 'Fontsize', fs2, 'FontWeight','bold');
end

exportgraphics(fig1,['../Figures/', fig1_name '.pdf'],'BackgroundColor','white','ContentType','image', 'Resolution', 300)

%% save data for Stark-fitting
dt = mean(diff(time_cc));
ind_time = find(time_cc > -time_avg_width*dt-10*eps & time_cc < time_avg_width*dt+10*eps); %find(wave > 400 & wave < 650); %find(wave > 450 & wave < 540); %find(wave > 470 & wave < 503);
selected_time_data = mean(TAS_data_2DM_cor(:, ind_time), 2);

disp(['Calculate the time average over ' num2str((time_avg_width*2+1)*dt) 'ps']);

wavenumber = 1./(wave*1e-9*1e2); %in 1/cm
absorbance = -log10(selected_time_data+1);
ind_tosave =  find(wave < 650 & wave > 390);
extinct = horzcat(wavenumber(ind_tosave), absorbance(ind_tosave));

if save_data == 1
    Samplename = erase(dataname,".txt"); %used for storing 1D data
    folder_path = ['../RawData/' measurementday '/']; %used for storing 1D data
    save_path = [folder_path, 'ExtractedTA_1D/', Samplename];
    save(save_path, 'extinct', '-tabs');
end
