% clear
clear variables;
close all;
clc;
%% physical constants
e = 1.60217662e-19;             %elementary charge (in C)
hbar = 6.62607015e-34/(2*pi);   %reduced Planck constant (in Js)
me = 9.109384e-31;              %electron rest mass (in kg)
c_light = 299792458;            %Speed of light in vacuum (in m/s)
kB = 1.380649e-23;              %Boltzmann constant (in J/K)
mu0 = 4*pi*1e-7;                %premeability of free space (in N/A^2)
e0 = 1/(mu0*c_light^2);         %permittivity of free space (in F/m)
%% setup figure
fs = 12; %fontsize
linewidth = 2;
set(groot, 'defaultAxesTickLabelInterpreter','tex');
set(groot, 'defaultLegendInterpreter','tex');
set(groot, 'defaultTextInterpreter','tex');
set(groot, 'defaultAxesFontSize',fs);
set(groot, 'defaultTextFontSize',fs);
set(groot, 'defaultAxesLabelFontSize', fs);
set(groot, 'defaultAxesLabelFontSizeMultiplier', 1);
set(groot, 'defaultLineLineWidth', linewidth);
set(groot, 'defaultAxesLineWidth', linewidth);

fig1_name = 'Test';
fig1 = figure('Name', fig1_name, 'Position', [10, 50, 400, 300]);

datapath_savefigure = '..e/Figures';
%% load data
%sample_id = 6; %set to 5 (toluene par. pol.), 6 (toluene per. pol)
for sample_id = [5, 6]

    mycolormap = diverging_map(linspace(0,1,100),[0,0,1],[1,0,0]);
    mycolormap2 = diverging_map(linspace(0,1,100),[1,1,1],[0.6,0,0]);

    % load EO-signal
    data_path = '../RawData/20220308/EO_signal_diffConfiguration/';
    data11 = dlmread([data_path, 'time0205date090322_initCond_withQuartz.txt']);
    inc = 75; %considered increments between scan positions
    dt = inc*2*0.1e-6/c_light;
    time_eo = (0:length(data11(:,2))-1)*dt*1e12;
    signal_eo = data11(:,2) - mean(data11(:,2));
    signal_eo = signal_eo/max(abs(signal_eo));

    % get data sample
    if sample_id == 1
        measurementday = '20210908';
        dataname = '00_55_23 Sam1  Cuvette per_transient.txt';
    elseif sample_id == 2
        measurementday = '20210909';
        dataname = '22_57_24 Sam2 par_transient.txt';
    elseif sample_id == 3
        measurementday = '20210908';
        dataname = '23_00_53 Sam1  Cuvette par_transient.txt';
    elseif sample_id == 4
        measurementday = '20210909';
        dataname = '01_20_56 Sam2 per_transient.txt';
    elseif sample_id == 5
        measurementday = '20220304';
        dataname = '01_24_01 Toluene_par_transient.txt';
    elseif sample_id == 6
        measurementday = '20220304';
        dataname = '02_25_13 Toluene_per_transient.txt';
    end



    titletext = strrep(erase(dataname,".txt"), '_', '\_');

    chirp_correction_case = 1; %set to 1 for Cuvette and 2 for Capillary
    time_avg_width = 0; 
    rem_bg = 25; 

    [wave, deltaA, time_cc, TAS_data_2DM] = get_deltaA_from_TASdata(measurementday, dataname, time_avg_width, rem_bg, chirp_correction_case);
    if sample_id == 6 %for this sample, the automatic zero-time finding is not working
        time_cc = time_cc-0.5;
    end
    %no background correction
    TAS_data_2DM_cor = TAS_data_2DM;
    switch sample_id
        case 5
            TAS_data_2DM_cor_par = TAS_data_2DM_cor;
            time_cc_par = time_cc;
            wave_par = wave;
        case 6
            TAS_data_2DM_cor_per = TAS_data_2DM_cor;
            time_cc_per = time_cc;
            wave_per = wave;
    end
end

%% generate figure
set(fig1, 'Position', [10 61 500 400]);
fig1_name = 'Toluene_ResultOverview_bothpol';
figure(fig1); clf;
for sample_id = [5,6]
    switch sample_id
        case 5
            TAS_data_2DM_cor = TAS_data_2DM_cor_par;
            time_cc = time_cc_par;
            wave = wave_par;
            titletext = 'Parallel orientation';
        case 6
            TAS_data_2DM_cor = TAS_data_2DM_cor_per;
            time_cc = time_cc_per;
            wave = wave_per;
            titletext = 'Perpendicular orientation';
    end

    signal_lim_scale = 0.6;
    wl_lim = [370, 670];
    col_lim = [-150, 150]*signal_lim_scale;
    signal_lim = [-30, 60];
    eo_scale = 110*signal_lim_scale;
    eo_timeoffset = 0;
    ind_bg = find(wave > 390 & wave < 650);
    ind_wave = find(wave > 500 & wave < 550); 
    time_avg_width = 3;
    OD_abs_mean_offset = 15;
    col_wave = [0.8, 0.4, 0.2];
    col_bg = [0.4, 0.2, 0.8];
    col_signal = [0.2, 0.8, 0.4];
    deltaA_ticks = [-150,0, 150]*signal_lim_scale;

    time_lim = [-1.5, 1];


    if sample_id == 5
        subplot(3,2,[1,3]);
    elseif sample_id == 6
        subplot(3,2,[2,4]);
    end
    %this points are in the middle of each rectangle
    dt = mean(diff(time_cc));
    time_cc_plot = time_cc - dt/2;
    wave_plot = wave;
    wave_plot(1) = wave(1)-(wave(1)-wave(2));
    for counter = 2:length(wave)
        wave_plot(counter) = wave(counter)-(wave(counter-1)-wave(counter));
    end
    hp = pcolor(time_cc_plot, wave_plot, -log10(TAS_data_2DM_cor+1)*1e6);


    hp.EdgeColor = 'none';
    set(gca, 'ydir', 'normal')
    colormap(mycolormap);
    if sample_id == 6
        hcol = colorbar;
        hcol.Location = 'manual';
        hcol.Position = [0.9275 0.4125 0.0246 0.5150];
        hcolTitle = get(hcol,'title');
        set(hcolTitle,'string','\Delta A (\muOD)', 'Rotation', 90);
        hcolTitle.Position = [31.1500 108.2000 0];
        hcol.Ticks = [col_lim(1), col_lim(2)];
    end
    caxis(col_lim);
    title(titletext)

    if sample_id == 5
        ylabel('Wavelength (nm)');
    end

    xlim(time_lim);
    ylim([wl_lim])


    % spectra
    dt = mean(diff(time_cc));
    ind_time = find(time_cc > -time_avg_width*dt & time_cc < time_avg_width*dt); %find(wave > 400 & wave < 650); %find(wave > 450 & wave < 540); %find(wave > 470 & wave < 503);
    OD_spec_mean = mean(-log10(TAS_data_2DM_cor(:, ind_time)+1)*1e6, 2);

    %add rectangle box
    t1 = time_cc(ind_time(1));
    t2 = time_cc(ind_time(end));
    wl1 = wl_lim(1);
    wl2 = wl_lim(2);

    t1 = time_lim(1);
    t2 = time_lim(2);
    wl1 = wave(ind_wave(end));
    wl2 = wave(ind_wave(1));

    t1 = time_lim(1);
    t2 = time_lim(2);
    wl1 = wave(ind_bg(end));
    wl2 = wave(ind_bg(1));
    h_box2 = rectangle('Position', [t1, wl1, t2-t1, wl2-wl1], 'EdgeColor',col_bg, 'Linewidth', 2, 'LineStyle', ':');

    box on;
    set(gca, 'Layer', 'top');

    % time signals
    OD_mean = mean(-log10(TAS_data_2DM_cor(ind_bg,:)+1)*1e6, 1);
    OD_abs_mean = mean(abs(-log10(TAS_data_2DM_cor(ind_wave,:)+1))*1e6, 1);

    if sample_id == 5
        subplot(3,2,[5]);
    elseif sample_id == 6
        subplot(3,2,[6]);
    end
    h_ODmean = plot(time_cc, OD_mean, 'Color', col_bg);
    if sample_id == 5
        ylabel('\Delta A (\muOD)');
    end
    grid on;
    xlim(time_lim);
    ylim(signal_lim);
    xlabel('Time (ps)');

    box on;
    set(gca, 'Layer', 'top');

end
set(findall(gcf,'-property','FontSize'),'FontSize',fs)

fs2 = 16;
h1 = text(-4.7, 280, 'a', 'Fontsize', fs2, 'FontWeight','bold');
h2 = text(-1.4, 280, 'b', 'Fontsize', fs2, 'FontWeight','bold');
h3 = text(-4.7, 47, 'c', 'Fontsize', fs2, 'FontWeight','bold');
h4 = text(-1.4, 47, 'd', 'Fontsize', fs2, 'FontWeight','bold');

exportgraphics(fig1,['../Figures/', fig1_name '.pdf'],'BackgroundColor','white','ContentType','image', 'Resolution', 300)
