% clear
clear variables;
close all;
clc;
%% physical constants
e = 1.60217662e-19;             %elementary charge (in C)
hbar = 6.62607015e-34/(2*pi);   %reduced Planck constant (in Js)
me = 9.109384e-31;              %electron rest mass (in kg)
c_light = 299792458;            %Speed of light in vacuum (in m/s)
kB = 1.380649e-23;              %Boltzmann constant (in J/K)
mu0 = 4*pi*1e-7;                %premeability of free space (in N/A^2)
e0 = 1/(mu0*c_light^2);         %permittivity of free space (in F/m)
%% setup figure
fs = 12; %fontsize
linewidth = 2;
set(groot, 'defaultAxesTickLabelInterpreter','tex');
set(groot, 'defaultLegendInterpreter','tex');
set(groot, 'defaultTextInterpreter','tex');
set(groot, 'defaultAxesFontSize',fs);
set(groot, 'defaultTextFontSize',fs);
set(groot, 'defaultAxesLabelFontSize', fs);
set(groot, 'defaultAxesLabelFontSizeMultiplier', 1);
set(groot, 'defaultLineLineWidth', linewidth);
set(groot, 'defaultAxesLineWidth', linewidth);

fig1_name = 'Test';
fig1 = figure('Name', fig1_name, 'Position', [10, 50, 400, 300]);

datapath_savefigure = '../Figures';

%% load data
%sample_id = 6; %set to 5 (toluene par. pol.), 6 (toluene per. pol)
for case_id = [1]

    mycolormap = diverging_map(linspace(0,1,100),[0,0,1],[1,0,0]);
    mycolormap2 = diverging_map(linspace(0,1,100),[1,1,1],[0.6,0,0]);
    % load EO-signal
    data_path = '../RawData/20220308/EO_signal_diffConfiguration/';
    data11 = dlmread([data_path, 'time0205date090322_initCond_withQuartz.txt']);
    inc = 75; %considered increments between scan positions
    dt = inc*2*0.1e-6/c_light;
    time_eo = (0:length(data11(:,2))-1)*dt*1e12;
    signal_eo = data11(:,2) - mean(data11(:,2));
    signal_eo = signal_eo/max(abs(signal_eo));

    %% get data sample
    if case_id == 1
        measurementday = '20210908';
        dataname = '00_55_23 Sam1  Cuvette per_transient.txt';
    elseif case_id == 2
        measurementday = '20210909';
        dataname = '22_57_24 Sam2 par_transient.txt';
    elseif case_id == 3
        measurementday = '20210908';
        dataname = '23_00_53 Sam1  Cuvette par_transient.txt';
    elseif case_id == 4
        measurementday = '20210909';
        dataname = '01_20_56 Sam2 per_transient.txt';
    elseif case_id == 5
        measurementday = '20220304';
        dataname = '01_24_01 Toluene_par_transient.txt';
    elseif case_id == 6
        measurementday = '20220304';
        dataname = '02_25_13 Toluene_per_transient.txt';
    end

    titletext = strrep(erase(dataname,".txt"), '_', '\_');

    
    time_avg_width = 0; 
    rem_bg = 25; 

    chirp_correction_case = 1; %set to 1 for Cuvette and 2 for Capillary
    [wave_withGVG, deltaA_withGVG, time_cc_withGVG, TAS_data_2DM_withGVG] = get_deltaA_from_TASdata(measurementday, dataname, time_avg_width, rem_bg, chirp_correction_case);
    
    chirp_correction_case = 0; %set to 1 for Cuvette and 2 for Capillary (to 0 for no gvd-correction)
    global fit_3rd
    [wave_noGVD, deltaA_noGVD, time_cc_noGVD, TAS_data_2DM_noGVD] = get_deltaA_from_TASdata(measurementday, dataname, time_avg_width, rem_bg, chirp_correction_case);

    if case_id == 6 %for this sample, the automatic zero-time finding is not working
        time_cc = time_cc-0.5;
    end
    
end
%% generate figure 

set(fig1, 'Position', [10 61 500 260]);
fig1_name = 'GVD_correction';
figure(fig1); clf;
for case_id = [1,2]
    switch case_id
        case 1
            TAS_data_2DM_cor = TAS_data_2DM_noGVD;
            time_cc = time_cc_noGVD;
            wave = wave_noGVD;
            titletext = 'Without GVD correction';
        case 2
            TAS_data_2DM_cor = TAS_data_2DM_withGVG;
            time_cc = time_cc_withGVG;
            wave = wave_withGVG;
            titletext = 'With GVD correction';
    end

    signal_lim_scale = 800/150;
    wl_lim = [370, 670];
    col_lim = [-150, 150]*signal_lim_scale;
    signal_lim = [-30, 60]; 
    eo_scale = 110*signal_lim_scale;
    eo_timeoffset = 0;
    ind_bg = find(wave > 390 & wave < 650);
    ind_wave = find(wave > 500 & wave < 550); 
    time_avg_width = 3;
    OD_abs_mean_offset = 15;
    col_wave = [0.8, 0.4, 0.2];
    col_bg = [0.4, 0.2, 0.8];
    col_signal = [0.2, 0.8, 0.4];
    deltaA_ticks = [-150,0, 150]*signal_lim_scale;

    time_lim = [-3, 1.8];


    if case_id == 1
        subplot(1,2,[1]); hold on;
    elseif case_id == 2
        subplot(1,2,[2]); hold on;
    end

    %this points are in the middle of each rectangle
    dt = mean(diff(time_cc));
    time_cc_plot = time_cc - dt/2;
    wave_plot = wave;
    wave_plot(1) = wave(1)-(wave(1)-wave(2));
    for counter = 2:length(wave)
        wave_plot(counter) = wave(counter)-(wave(counter-1)-wave(counter));
    end
    hp = pcolor(time_cc_plot, wave_plot, -log10(TAS_data_2DM_cor+1)*1e6);

    hp.EdgeColor = 'none';
    set(gca, 'ydir', 'normal')
    colormap(mycolormap);
    if case_id == 2
        hcol = colorbar;
        hcol.Location = 'manual';
        hcol.Position = [0.9180 0.1500 0.0200 0.7775];
        hcolTitle = get(hcol,'title');
        set(hcolTitle,'string','\Delta A (\muOD)', 'Rotation', 90);
        hcolTitle.Position = [31.1500 108.2000 0];
        hcol.Ticks = [col_lim(1), col_lim(2)];
    end
    caxis(col_lim);
    title(titletext)

    

    if case_id == 1
        ylabel('Wavelength (nm)');
    end

   set(gca, 'Layer', 'top');

   ylim(wl_lim);

   %add gvd-curve
   if case_id == 1
       plot(fit_3rd*1e-3-0.6, wave_plot, 'Color', 'k', 'LineWidth',2);
       xlim([-3, 1]+0.2);
   else
       plot([0 0], [200, 800], 'Color', 'k', 'LineWidth',2);
       xlim([-3, 1]+0.8);
   end
   xlabel('Time (ps)');
   box on;
end
set(findall(gcf,'-property','FontSize'),'FontSize',fs)

fs2 = 16;
h1 = text(-7, 600, 'a', 'Fontsize', fs2, 'FontWeight','bold');
h2 = text(-1.6, 600, 'b', 'Fontsize', fs2, 'FontWeight','bold');

%% save figure
exportgraphics(fig1,['../Figures/', fig1_name '.pdf'],'BackgroundColor','white','ContentType','image', 'Resolution', 300)
