% clear
clear variables;
close all;
clc;
%% physical constants
e = 1.60217662e-19;             %elementary charge (in C)
hbar = 6.62607015e-34/(2*pi);   %reduced Planck constant (in Js)
me = 9.109384e-31;              %electron rest mass (in kg)
c_light = 299792458;            %Speed of light in vacuum (in m/s)
kB = 1.380649e-23;              %Boltzmann constant (in J/K)
mu0 = 4*pi*1e-7;                %premeability of free space (in N/A^2)
e0 = 1/(mu0*c_light^2);         %permittivity of free space (in F/m)
%% setup figure
fs = 12; %fontsize
linewidth = 2;
set(groot, 'defaultAxesTickLabelInterpreter','tex'); 
set(groot, 'defaultLegendInterpreter','tex');
set(groot, 'defaultTextInterpreter','tex');
set(groot, 'defaultAxesFontSize',fs);
set(groot, 'defaultTextFontSize',fs);
set(groot, 'defaultAxesLabelFontSize', fs);
set(groot, 'defaultAxesLabelFontSizeMultiplier', 1);
set(groot, 'defaultLineLineWidth', linewidth);
set(groot, 'defaultAxesLineWidth', linewidth);

fig1_name = 'StarkFits_TTF';
fig1 = figure('Name', fig1_name, 'Position', [22 67 625 520]);
datapath_savefigure = '../Figures'; 
%% get AC data
AC_datapath = '../RawData/StarkFit_DataSets/';
sample1_filename = 'Sample1_AC/sam1_ac_test4';
sample2_filename = 'Sample2_AC/sam2_ac_test3';
for sample_id = 1:2
    if sample_id == 1
        alpha_pol = [90, 0]; %polarizations (in deg) (90deg is horizontal)
        alpha_inc = [45, 45]; %incidence angle (in deg)
        sample_filename = sample1_filename;
        titletext = 'Anthanthrene';
        legpos = 'northeast';
        xrange = [440, 550];
    elseif sample_id == 2
        alpha_pol = [90, 0]; %polarizations (in deg) (90deg is horizontal)
        alpha_inc = [37, 37]; %incidence angle (in deg)
        sample_filename = sample2_filename;
        titletext = 'TTF-BTD';
        legpos = 'northeast';
        xrange = [400, 650];
    end

    %get angle between probe and stark electric field
    for counter = 1:2
        if alpha_pol(counter) == 90
            n1 = 1.2; n2 = 1.5; %this are the same parameters as used in the Stark-fitting
            theta2 = asind(n1/n2*sind(alpha_inc(counter)));
            beta(counter) = round(90-theta2);
        elseif alpha_pol(counter) == 0
            beta(counter) = 90;
        else
            disp('Can not handle this polarization angle. Please check.')
        end
    end

    data_absfit = dlmread([AC_datapath, sample_filename, '_ABSFIT.txt'], '',1,0);
    data_stkfit1 = dlmread([AC_datapath, sample_filename, '_STKFIT1.txt'], '',1,0);
    data_stkfit2 = dlmread([AC_datapath, sample_filename, '_STKFIT2.txt'], '',1,0);
    
    load([AC_datapath, sample_filename, '.mat']);

    if sample_id == 1
        data_absfit_sam1_ac = data_absfit;
        data_stkfit1_sam1_ac = data_stkfit1;
        data_stkfit2_sam1_ac = data_stkfit2;
        beta_sam1_ac = beta;
        obj_sam1_ac = copy(obj);
    else
        data_absfit_sam2_ac = data_absfit;
        data_stkfit1_sam2_ac = data_stkfit1;
        data_stkfit2_sam2_ac = data_stkfit2;
        beta_sam2_ac = beta;
        obj_sam2_ac = copy(obj);
    end
end

%% get THz data
THz_datapath = '../RawData/StarkFit_DataSets/';
sample1_filename = 'Sample1_Cuvette/sam1_cuvette_test13';
sample2_filename = 'Sample2_Cuvette/sam2_cuvette_test13';
for sample_id = 1:2
    if sample_id == 1
        alpha_pol = [90, 0]; %polarizations (in deg) (90deg is horizontal)
        alpha_inc = [90, 0]; %incidence angle (in deg)
        sample_filename = sample1_filename;
        titletext = 'Anthanthrene';
        legpos = 'northeast';
        xrange = [440, 550];
    elseif sample_id == 2
        alpha_pol = [90, 0]; %polarizations (in deg) (90deg is horizontal)
        alpha_inc = [90, 0]; %incidence angle (in deg)
        sample_filename = sample2_filename;
        titletext = 'TTF-BTD';
        legpos = 'northeast';
        xrange = [400, 650];
    end

    data_absfit = dlmread([THz_datapath, sample_filename, '_ABSFIT.txt'], '',1,0);
    data_stkfit1 = dlmread([THz_datapath, sample_filename, '_STKFIT1.txt'], '',1,0);
    data_stkfit2 = dlmread([THz_datapath, sample_filename, '_STKFIT2.txt'], '',1,0);

    %get angle between probe and stark electric field
    for counter = 1:2
        if alpha_pol(counter) == 90
            beta(counter) = 90-alpha_inc(counter);
        elseif alpha_pol(counter) == 0
            beta(counter) = 90;
        else
            disp('Can not handle this polarization angle. Please check.')
        end
    end

    load([THz_datapath, sample_filename, '.mat']);

    if sample_id == 1
        data_absfit_sam1_thz = data_absfit;
        data_stkfit1_sam1_thz = data_stkfit1;
        data_stkfit2_sam1_thz = data_stkfit2;
        beta_sam1_thz = beta;
        obj_sam1_thz = copy(obj);
    else
        data_absfit_sam2_thz = data_absfit;
        data_stkfit1_sam2_thz = data_stkfit1;
        data_stkfit2_sam2_thz = data_stkfit2;
        beta_sam2_thz = beta;
        obj_sam2_thz = copy(obj);        
    end
end
%% plot figure
for sample_counter =  1:2
    figure(fig1); clf;
    set(fig1, 'Position', [22 67 625 325*3/2])
    col_ac1 = [0, 0.4, 0.7];
    col_ac2 = col_ac1 + [0.3, 0.3, 0.3]; 
    col_thz1 = [0.70, 0.2, 0.0];
    col_thz2 = col_thz1 + [0.3, 0.3, 0.3];  
    A_chi_scaling = 10;
    switch sample_counter 
        case 1
            fig1_name = 'StarkFits_Anthanthrene';
            titletext = 'Anthanthrene';
            deltaEps_plotoffset = 3000;
            xax_lim = [400, 550];
            yax1_lim = [0, 1.1e5];
            yax2_lim = [-4000, 2500+deltaEps_plotoffset];
            yax3_lim = [-3000, 2500];
            data_absfit_thz = data_absfit_sam1_thz;
            data_absfit_ac = data_absfit_sam1_ac;
            data_stkfit1_ac = data_stkfit1_sam1_ac;
            data_stkfit2_ac = data_stkfit2_sam1_ac;
            data_stkfit1_thz = data_stkfit1_sam1_thz;
            data_stkfit2_thz = data_stkfit2_sam1_thz;
            beta_sam_ac = beta_sam1_ac;
            beta_sam_thz = beta_sam1_thz;
            
        case 2
            fig1_name = 'StarkFits_TTF-BTD';
            titletext = 'TTF-BTD';
            deltaEps_plotoffset = 200;
            xax_lim = [400, 650];
            yax1_lim = [0, 3e4];
            yax2_lim = [-610, 250+deltaEps_plotoffset];
            yax3_lim = [-300, 200];
            data_absfit_thz = data_absfit_sam2_thz;
            data_absfit_ac = data_absfit_sam2_ac;
            data_stkfit1_ac = data_stkfit1_sam2_ac;
            data_stkfit2_ac = data_stkfit2_sam2_ac;
            data_stkfit1_thz = data_stkfit1_sam2_thz;
            data_stkfit2_thz = data_stkfit2_sam2_thz;
            beta_sam_ac = beta_sam2_ac;
            beta_sam_thz = beta_sam2_thz;
    end
    subplot(3,2,1); hold on;
    plot(1./data_absfit_ac(:,1)*1e9*1e-2, data_absfit_ac(:,2), '.', 'Color', col_ac1, 'Markersize', 12)
    plot(1./data_absfit_ac(:,1)*1e9*1e-2, data_absfit_ac(:,3), '-', 'Color', 'k')

    ylabel({'Molar attenuation' 'coefficient \epsilon (M^{-1} cm^{-1})'});

    grid on;
    title('Conv. Stark');
    xlim(xax_lim );
    ylim(yax1_lim);
    box on;

    subplot(3,2,2); hold on
    plot(1./data_absfit_thz(:,1)*1e9*1e-2, data_absfit_thz(:,2), '.', 'Color', col_thz1, 'Markersize', 10)
    plot(1./data_absfit_thz(:,1)*1e9*1e-2, data_absfit_thz(:,3), '-', 'Color', 'k')
    grid on;
    title('THz Stark');
    xlim(xax_lim );
    ylim(yax1_lim);
    box on;

    subplot(3,2,3); hold on;
    plot(1./data_stkfit1_ac(:,1)*1e9*1e-2, data_stkfit1_ac(:,2), '.', 'Color', col_ac1, 'Markersize', 12)
    plot(1./data_stkfit2_ac(:,1)*1e9*1e-2, data_stkfit2_ac(:,2)+deltaEps_plotoffset, '.',  'Color', col_ac2, 'Markersize', 12)

    plot(1./data_stkfit1_ac(:,1)*1e9*1e-2, data_stkfit1_ac(:,3), '-', 'Color', 'k')
    plot(1./data_stkfit2_ac(:,1)*1e9*1e-2, data_stkfit2_ac(:,3)+deltaEps_plotoffset, '-',  'Color', 'k')

    legend({['Conv. (\chi = ' num2str(beta_sam_ac(1)) '°)'], ...
        ['Conv. (\chi = ' num2str(beta_sam_ac(2)) '°)']}, 'Location', 'southwest')
    ylim(yax2_lim);
    xlim(xax_lim);
    grid on; box on;

    hylabel = ylabel({'\Delta \epsilon (M^{-1} cm^{-1})'});

    subplot(3,2,4); hold on;
    plot(1./data_stkfit1_thz(:,1)*1e9*1e-2, data_stkfit1_thz(:,2), '.', 'Color', col_thz1, 'Markersize', 12);
    plot(1./data_stkfit2_thz(:,1)*1e9*1e-2, data_stkfit2_thz(:,2)+deltaEps_plotoffset, '.', 'Color', col_thz2, 'Markersize', 12);

    plot(1./data_stkfit1_thz(:,1)*1e9*1e-2, data_stkfit1_thz(:,3), '-', 'Color', 'k');
    plot(1./data_stkfit2_thz(:,1)*1e9*1e-2, data_stkfit2_thz(:,3)+deltaEps_plotoffset, '-', 'Color', 'k');

    legend({['THz (\chi = ' num2str(beta_sam_thz(1)) '°)'], ...
       ['THz (\chi = ' num2str(beta_sam_thz(2)) '°)']}, 'Location', 'southwest')

 
    grid on; box on;
    xlim(xax_lim);
    ylim(yax2_lim);

    subplot(3,2,5); hold on;
    plot(1./data_stkfit2_ac(:,1)*1e9*1e-2, A_chi_scaling*data_stkfit2_ac(:,4), '-', 'Color', col_ac2);
    plot(1./data_stkfit2_ac(:,1)*1e9*1e-2, data_stkfit2_ac(:,5), '--', 'Color', col_ac2);
    plot(1./data_stkfit2_ac(:,1)*1e9*1e-2, data_stkfit2_ac(:,6), ':', 'Color', col_ac2);
    legtext = {['$\widehat{A}_{\chi}$ ($\times$' num2str(A_chi_scaling) ')'], '$\widehat{B}_{\chi}$', '$\widehat{C}_{\chi}$'}; 
    legend(legtext, 'Interpreter', 'latex', 'Location', 'southwest', 'NumColumns', 3);
    xlim(xax_lim);
    ylim(yax3_lim);
    grid on; box on;
    xlabel('Wavelength (nm)')
    hylabel = ylabel({'\Delta \epsilon (M^{-1} cm^{-1})'});

    subplot(3,2,6); hold on;
    plot(1./data_stkfit2_thz(:,1)*1e9*1e-2, A_chi_scaling*data_stkfit2_thz(:,4), '-', 'Color', col_thz2);
    plot(1./data_stkfit2_thz(:,1)*1e9*1e-2, data_stkfit2_thz(:,5), '--', 'Color', col_thz2);
    plot(1./data_stkfit2_thz(:,1)*1e9*1e-2, data_stkfit2_thz(:,6), ':', 'Color', col_thz2);
    legend(legtext, 'Interpreter', 'latex', 'Location', 'southwest', 'NumColumns', 3);
    grid on; box on;
    xlim(xax_lim);
    ylim(yax3_lim);
    xlabel('Wavelength (nm)')


    figure(fig1);
    set(findall(gcf,'-property','FontSize'),'FontSize',fs)

    fs2 = 16;
    switch sample_counter
        case 1
            xlabel_pos = 410;
            ylabel_pos1 = 8e4;
            ylabel_pos2 = 4500;
            ylabel_pos3 = 1500;
        case 2
            xlabel_pos = 410;
            ylabel_pos1 = 2.5e4;
            ylabel_pos2 = 350;
            ylabel_pos3 = 120;
    end
    subplot(3,2,1);
    h1 = text( xlabel_pos, ylabel_pos1, 'a', 'Fontsize', fs2, 'FontWeight','bold');
    subplot(3,2,2);
    h1 = text( xlabel_pos, ylabel_pos1, 'b', 'Fontsize', fs2, 'FontWeight','bold');
    subplot(3,2,3);
    h1 = text( xlabel_pos, ylabel_pos2, 'c', 'Fontsize', fs2, 'FontWeight','bold');
    subplot(3,2,4);
    h1 = text( xlabel_pos, ylabel_pos2, 'd', 'Fontsize', fs2, 'FontWeight','bold');
    subplot(3,2,5);
    h1 = text( xlabel_pos, ylabel_pos3, 'e', 'Fontsize', fs2, 'FontWeight','bold');
    subplot(3,2,6);
    h1 = text( xlabel_pos, ylabel_pos3, 'f', 'Fontsize', fs2, 'FontWeight','bold');

    exportgraphics(fig1,[datapath_savefigure '/', fig1_name '.pdf'],'BackgroundColor','white','ContentType','image', 'Resolution', 300)

end

%% error calculation

for sample_counter = 1:2
    for measurement_id = 1:2
        if sample_counter == 1 && measurement_id == 1
            obj = copy(obj_sam1_ac);
            case_text = 'Anthanthrene, AC:';
            f_loc = 1.33; %see p.42 in myNotebook 4
            sigma_E_rel = 0e-2; %relative error in e-field strenght (for AC this is just an assumption)
        elseif sample_counter == 1 && measurement_id == 2
            obj = copy(obj_sam1_thz);
            case_text = 'Anthanthrene, THz:';
            f_loc = 1.29; %see p.42 in myNotebook 4
            sigma_E_rel = 325/2828; %relative error in e-field strenght
        elseif sample_counter == 2 && measurement_id == 1
            obj = copy(obj_sam2_ac);
            case_text = 'TTF-BTD, AC:';
            f_loc = 1.30; %see p.42 in myNotebook 4
            sigma_E_rel = 0e-2; %relative error in e-field strenght (for AC this is just an assumption)
        elseif sample_counter == 2 && measurement_id == 2
            obj = copy(obj_sam2_thz);
            case_text = 'TTF-BTD, THz:';
            f_loc = 1.26; %see p.42 in myNotebook 4
            sigma_E_rel = 325/2828; %relative error in e-field strenght
        end


        obj.mc.rf(:,3,:) = abs(obj.mc.rf(:,3,:));
        obj.mc.rf(:,4,:) = abs(obj.mc.rf(:,4,:));


        %for more robust chilim estimation
        chi_med = median(obj.mc.chi2);
        chi_mad = mad(obj.mc.chi2, 1);
        chilim = log10(chi_med+10*chi_mad); 


        obj.mc.ri(:,:,(log10(obj.mc.chi2)) > chilim) = [];
        obj.mc.rf(:,:,(log10(obj.mc.chi2)) > chilim) = [];
        obj.mc.chi2((log10(obj.mc.chi2)) > chilim) = [];

        num_regected = obj.opts.MCiter-size(obj.mc.rf,3);

        aTooBig = squeeze(max(max(obj.mc.af > 1e-20)));

        obj.mc.ri(:,:,aTooBig) = [];
        obj.mc.rf(:,:,aTooBig) = [];

        disp(case_text)
        disp(['reject ' num2str(num_regected/obj.opts.MCiter*100) '% of all iterations.']);

        indx = [1:max(obj.bands)];

        meanTrAlpha = mean((obj.mc.rf(:,1,:)),3)';
        stdTrAlpha = std((obj.mc.rf(:,1,:)),0,3)';

        meanProjAlpha = mean((obj.mc.rf(:,2,:)),3)';
        stdProjAlpha = std((obj.mc.rf(:,2,:)),0,3)';
        meanDeltaMu = mean((obj.mc.rf(:,3,:)),3)';
        stdDeltaMu = std((obj.mc.rf(:,3,:)),0,3)';
        meanZeta = mean((obj.mc.rf(:,4,:)),3)';
        stdZeta = std((obj.mc.rf(:,4,:)),0,3)';

        %local field correction
        meanTrAlpha = meanTrAlpha/f_loc.^2;
        stdTrAlpha = stdTrAlpha/f_loc.^2;
        meanProjAlpha = meanProjAlpha/f_loc.^2;
        stdProjAlpha = stdProjAlpha/f_loc.^2;
        meanDeltaMu = meanDeltaMu/f_loc.^1;
        stdDeltaMu = stdDeltaMu/f_loc.^1;
        meanZeta = meanZeta;
        stdZeta = stdZeta;

        %include E-field errors (Gaussian error propagation)
        stdTrAlpha = sqrt(stdTrAlpha^2 + (2*meanTrAlpha*sigma_E_rel)^2);
        stdProjAlpha = sqrt(stdProjAlpha^2 + (2*meanProjAlpha*sigma_E_rel)^2);
        stdDeltaMu = sqrt(stdDeltaMu^2 + (meanDeltaMu*sigma_E_rel)^2);
        stdZeta = stdZeta;

        str1 = sprintf('Tr(Da)[0%i]: %#4.3g +/- %4.3g\n',[indx;meanTrAlpha;stdTrAlpha]);
        str2 = sprintf('m(Da)m[0%i]: %#4.3g +/- %4.3g\n',[indx;meanProjAlpha;stdProjAlpha]);
        str3 = sprintf('Delta mu[0%i]: %#4.3g +/- %4.3g\n',[indx;meanDeltaMu;stdDeltaMu]);
        str4 = sprintf('zeta[0%i]: %#4.3g +/- %4.3g\n',[indx;meanZeta;stdZeta]);

        str_tot = [str1, str2, str3, str4, sprintf('\n')];
        disp(str_tot);

    end
end

