% clear
clear variables;
close all;
clc;
%% physical constants
e = 1.60217662e-19;             %elementary charge (in C)
hbar = 6.62607015e-34/(2*pi);   %reduced Planck constant (in Js)
me = 9.109384e-31;              %electron rest mass (in kg)
c_light = 299792458;            %Speed of light in vacuum (in m/s)
kB = 1.380649e-23;              %Boltzmann constant (in J/K)
mu0 = 4*pi*1e-7;                %premeability of free space (in N/A^2)
e0 = 1/(mu0*c_light^2);         %permittivity of free space (in F/m)
%% setup figure
fs = 12; %fontsize
linewidth = 2;
set(groot, 'defaultAxesTickLabelInterpreter','tex');
set(groot, 'defaultLegendInterpreter','tex');
set(groot, 'defaultTextInterpreter','tex');
set(groot, 'defaultAxesFontSize',fs);
set(groot, 'defaultTextFontSize',fs);
set(groot, 'defaultAxesLabelFontSize', fs);
set(groot, 'defaultAxesLabelFontSizeMultiplier', 1);
set(groot, 'defaultLineLineWidth', linewidth);
set(groot, 'defaultAxesLineWidth', linewidth);

fig1_name = 'THzPulse_and_spectrum';
fig1 = figure('Name', fig1_name, 'Position', [10, 50, 400, 300]);

datapath_savefigure = '../Figures/';

mycolormap = diverging_map(linspace(0,1,100),[0,0,1],[1,0,0]);
mycolormap2 = diverging_map(linspace(0,1,100),[1,1,1],[0.6,0,0]);
%% load EO-signal
data_path = '../RawData/20220308/EO_signal_diffConfiguration/';
data11 = dlmread([data_path, 'time0205date090322_initCond_withQuartz.txt']);
inc = 75; %considered increments between scan positions
dt = inc*2*0.1e-6/c_light;
time_eo = (0:length(data11(:,2))-1)*dt;
signal_eo = data11(:,2) - mean(data11(:,2));
signal_eo = signal_eo/max(abs(signal_eo));
signal_eo = -1*signal_eo;
%% plot figure
time_shift = 3; %in ps
figure(fig1); clf; 
set(fig1, 'Position', [10 59 400 190])
subplot(1,2,1);
plot(time_eo*1e12-time_shift, signal_eo);
xlabel('Time (ps)');
ylabel('THz electric signal (normalized)');
grid on;
xlim([-3, 3]);
ylim([-0.8, 1.1]);


subplot(1,2,2);
[freq_eo, spec ] = myfft(time_eo, signal_eo);
plot(freq_eo*1e-12, abs(spec)./max(abs(spec)));
xlabel('Frequency (THz)');
ylabel('Spectral amplitude (normalized)');
grid on;
xlim([0, 2]);
ylim([0, 1.1]);

set(findall(gcf,'-property','FontSize'),'FontSize',fs)

%%
fs2 = 16;
h1 = text(-2.5, 1, 'a', 'Fontsize', fs2, 'FontWeight','bold');
h2 = text(0.1, 1, 'b', 'Fontsize', fs2, 'FontWeight','bold');

exportgraphics(fig1,[datapath_savefigure, fig1_name '.pdf'],'BackgroundColor','white','ContentType','image', 'Resolution', 300)
