function [wave, delta_absorbance, time_cc, TAS_data_2DM] = get_TASdata_fromPythonGui(measurementday, foldername, dataname, consec_scan, scan_id_vec, time_avg_width, rem_bg, chirp_correction_case )
%get absorbance change as function of wavelength from the TAS data.

%% physical constants
e = 1.60217662e-19;             %elementary charge (in C)
hbar = 6.62607015e-34/(2*pi);   %reduced Planck constant (in Js)
me = 9.109384e-31;              %electron rest mass (in kg)
c_light = 299792458;            %Speed of light in vacuum (in m/s)
kB = 1.380649e-23;              %Boltzmann constant (in J/K)
mu0 = 4*pi*1e-7;                %premeability of free space (in N/A^2)
e0 = 1/(mu0*c_light^2);         %permittivity of free space (in F/m)

%% sample information

data_path_whitelight = ['../RawData/' measurementday, '/SCcalibration_HWP15deg/'];

folder_path = ['../RawData/' measurementday '/'];

Samplename = dataname; 
time_cc = 0.0; 
selected_time = 2.18+0.003823-0.06618;
auto_select_time = 0; %1; %control variable
save_data = 0; %control variable


% control how to correct chirp
correct_chirp_wavelengthbased = 1;

remove_noisy_delay = 2; %case 0 for no removing, case 1 for usinge findpeaks, case 2 for using upper threshold

% default
mfnum2 = 1;
mfnum = 1;

mycolormap = diverging_map(linspace(0,1,100),[0,0,1],[1,0,0]);

cmin = 0.99995;cmax = 1.00005;
cmin2 = -0.0005;cmax2 = +0.0005;
Wavelength = 500; % selected wavelength (in nm) for signal change as function of time-delay

datafrom = [folder_path]; 
%% white light calibration
% read in data
%format: Pixel, Cam1 pumped, Cam1 unpumped, Cam2 pumped, Cam2 unpumped, Pumped Ratio, Unpumped Ratio, Transient Dual Cam, Transient Single Cam 
data400 = dlmread([data_path_whitelight, '400_GlazData.txt'], ',', 3, 0);
data415 = dlmread([data_path_whitelight, '415_GlazData.txt'], ',', 3, 0);
data488 = dlmread([data_path_whitelight, '488_GlazData.txt'], ',', 3, 0);
data536 = dlmread([data_path_whitelight, '536_GlazData.txt'], ',', 3, 0);
data542 = dlmread([data_path_whitelight, '542_GlazData.txt'], ',', 3, 0);
data577 = dlmread([data_path_whitelight, '577_GlazData.txt'], ',', 3, 0);
datafull = dlmread([data_path_whitelight, 'FullSpectra_GlazData.txt'], ',', 3, 0);

pixel = data400(:,1);
ind = 3; %set to 3 to use the unpumped Cam1 spectra
int_offset = 500; %only to avoid numerical issues when normalizing with low intensities close to 0
data400 = data400(:,ind)./(datafull(:,ind)+int_offset);
data415 = data415(:,ind)./(datafull(:,ind)+int_offset);
data488 = data488(:,ind)./(datafull(:,ind)+int_offset);
data536 = data536(:,ind)./(datafull(:,ind)+int_offset);
data542 = data542(:,ind)./(datafull(:,ind)+int_offset);
data577 = data577(:,ind)./(datafull(:,ind)+int_offset);

[pks, pxind400] = max(data400); %find peak and corresponding index
px400 = pixel(pxind400); %get the corresponding pixel number
weightligthCalib = [px400, 400]; %wavelength in nm

%get the multiple peaks from the 415 filter
filter415wavelength = flip([420.4, 615, 629.5, 650.9, 678.4, 712]);
[pks, pxind415] = findpeaks(data415, 'MinPeakHeight', 0.05, 'MinPeakDistance', 20, 'MinPeakWidth', 2);
if length(pxind415) ~= length(filter415wavelength)
    if length(pxind415) == length(filter415wavelength)-1
        filter415wavelength = filter415wavelength(2:end); %sometimes the last peak is missing
    else
        disp('Error in calibartion with 415-filter!');
        return;
    end
else %for fitting it is better not to use the last peak value
    filter415wavelength = filter415wavelength(2:end);
    pxind415 = pxind415(2:end);
end
for counter = 1:length(filter415wavelength)
    weightligthCalib = [weightligthCalib; [pixel(pxind415(counter))], filter415wavelength(counter)];   
end

[pks, pxind488] = max(data488);
px488 = pixel(pxind488); %get the corresponding pixel number
weightligthCalib = [weightligthCalib; [px488, 488.5]];

[pks, pxind536] = max(data536);
px536 = pixel(pxind536); %get the corresponding pixel number
weightligthCalib = [weightligthCalib; [px536, 536]];

[pks, pxind542] = max(data542);
px542 = pixel(pxind542); %get the corresponding pixel number
weightligthCalib = [weightligthCalib; [px542, 545.5]]; 
[pks, pxind577] = max(data577);
px577 = pixel(pxind577); %get the corresponding pixel number
weightligthCalib = [weightligthCalib; [px577, 577]]; 

%sort
[pixelnr, sorted_index] = sort(weightligthCalib(:,1));
weightligthCalib = [pixelnr, weightligthCalib(sorted_index,2)];

%linear fit
f_fit = fittype('m*x+q');
xfit = fit(weightligthCalib(:,1),weightligthCalib(:,2),f_fit,'StartPoint',[-1, 800]);
%quadratic fit
fit_ind = find(weightligthCalib(:,2) >= 480 & weightligthCalib(:,2) <= 660);
f_fit = fittype('a*x^2+m*x+q');
xfit = fit(weightligthCalib(fit_ind,1),weightligthCalib(fit_ind,2),f_fit,'StartPoint',[0, -1, 800]);

calib_wave(1) = xfit.q;
calib_wave(2) = xfit.m;
calib_wave(3) = xfit.a;

%calculate wavelength corresponding to each pixel
wave = pixel.^2*calib_wave(3)+pixel*calib_wave(2)+calib_wave(1); %for quadratic fitting


%% load TAS data
if consec_scan
    for counter = 1:length(scan_id_vec)
        data1_cur = dlmread([datafrom, foldername, '/', dataname, '_DoubleRef_' num2str(scan_id_vec(counter)) '.txt'], ',', 3,0);
        data2_cur = dlmread([datafrom, foldername, '/', dataname, '_SingleRef_' num2str(scan_id_vec(counter)) '.txt'], ',', 3,0);
        data3_cur = dlmread([datafrom, foldername, '/', dataname, '_DoubleRefv2_' num2str(scan_id_vec(counter)) '.txt'], ',', 3,0);
        data4_cur = dlmread([datafrom, foldername, '/', dataname, '_SingleRefv2_' num2str(scan_id_vec(counter)) '.txt'], ',', 3,0);
        if counter == 1
            data1_all = zeros(size(data1_cur,1), size(data1_cur, 2), length(scan_id_vec));
            data2_all = zeros(size(data1_cur,1), size(data1_cur, 2), length(scan_id_vec));
            data3_all = zeros(size(data1_cur,1), size(data1_cur, 2), length(scan_id_vec));
            data4_all = zeros(size(data1_cur,1), size(data1_cur, 2), length(scan_id_vec));
        end
        data1_all(:,:,counter) = data1_cur;
        data2_all(:,:,counter) = data2_cur;
        data3_all(:,:,counter) = data3_cur;
        data4_all(:,:,counter) = data4_cur;
    end
    data1 = mean(data1_all,3);
    data2 = mean(data2_all,3);
    data3 = mean(data3_all,3);
    data4 = mean(data4_all,3);
    data1_row_all = data1_all;
    data2_row_all = data2_all;
    data3_row_all = data3_all;
    data4_row_all = data4_all;
else
    data1 = dlmread([datafrom, dataname, '_DoubleRef.txt'], ',', 3,0);
    data2 = dlmread([datafrom, dataname, '_SingleRef.txt'], ',', 3,0);
    data3 = dlmread([datafrom, dataname, '_DoubleRefv2.txt'], ',', 3,0);
    data4 = dlmread([datafrom, dataname, '_SingleRefv2.txt'], ',', 3,0);
    data1_row_all = data1;
    data2_row_all = data2;
    data3_row_all = data3;
    data4_row_all = data4;

    data1_all = data1;
    data2_all = data2;
    data3_all = data3;
    data4_all = data4;
end
%% filter out bad scan lines 
if remove_noisy_delay == 1
    ind_pix = find(pixel < 800 & pixel > 400); %roi for detecting bad scan lines
    for scan_id = 1:length(scan_id_vec)
        data1 = data1_row_all(:,2:end,scan_id);

        mean_sig = mean(abs(data1(:,ind_pix)-1),2);
        [~, noisy_delay_ind] = findpeaks(mean_sig,'MinPeakHeight',1e-4, 'MinPeakDistance',1, 'MinPeakProminence',1e-4);        
        %manually check start and end of the signal 
        if mean_sig(1) > 1e-3
            noisy_delay_ind = [1; noisy_delay_ind];
        end
        if mean_sig(end) > 1e-3
            noisy_delay_ind = [noisy_delay_ind; length(mean_sig)];
        end
        disp([num2str(length(noisy_delay_ind)) ' noisy delay points were detected and are replaced by the mean value of the neighbors.'])
        for counter = 1:length(noisy_delay_ind)
            delay_ind = noisy_delay_ind(counter);
            if delay_ind == 1
                data1(delay_ind,:) = data1(delay_ind+1,:);
            elseif delay_ind == length(mean_sig)
                data1(delay_ind,:) = data1(delay_ind-1,:);
            else
                data1(delay_ind,:) = 0.5*( data1(delay_ind-1,:) + data1(delay_ind+1,:));
            end
        end
        data1_all(:,2:end,scan_id) = data1;

        subplot(2,1,1); 
        imagesc(data1_row_all(:,2:end,scan_id));
        colorbar; caxis([0.999, 1.001]);
        subplot(2,1,2); 
        imagesc(data1_all(:,2:end,scan_id));
        colorbar; caxis([0.999, 1.001]);
    end
end

if remove_noisy_delay == 2
    ind_pix = find(pixel < 800 & pixel > 400); %roi for detecting bad scan lines
    for scan_id = 1:length(scan_id_vec)
        data1 = data1_row_all(:,2:end,scan_id);
        data2 = data2_row_all(:,2:end,scan_id);
        data3 = data3_row_all(:,2:end,scan_id);
        data4 = data4_row_all(:,2:end,scan_id);

        mean_sig = abs(mean((abs(data1(:,ind_pix)-1)),2)- median(mean(abs(data1(:,ind_pix)-1),2)));

        noisy_delay_ind = find(mean_sig > 2e-4);   %for 5000triggers
        %manually check start and end of the signal         
        disp([num2str(length(noisy_delay_ind)) ' noisy delay points were detected and are replaced by zeros.'])
        for counter = 1:length(noisy_delay_ind)
            delay_ind = noisy_delay_ind(counter);           
            data1(delay_ind,:) = NaN; 
            data2(delay_ind,:) = NaN; 
            data3(delay_ind,:) = NaN; 
            data4(delay_ind,:) = NaN; 
        end
        data1_all(:,2:end,scan_id) = data1;
        data2_all(:,2:end,scan_id) = data2;
        data3_all(:,2:end,scan_id) = data3;
        data4_all(:,2:end,scan_id) = data4;
        
        num_noise(scan_id) = length(noisy_delay_ind); 
    end
end

if remove_noisy_delay == 0
    data1_all = data1_row_all;
    data2_all = data2_row_all;
    data3_all = data3_row_all;
    data4_all = data4_row_all;
end

 %% evaluate data (and combine single and double referencing)
 clear noise_level_single noise_level_dual noise_level_dualv2 noise_level_singlev2 noise_level_combined
 for scan_counter =  1:length(scan_id_vec) %length(scan_id_vec) %
     data1 = data1_all(:,:,scan_counter);
     data2 = data2_all(:,:,scan_counter);
     data3 = data3_all(:,:,scan_counter);
     data4 = data4_all(:,:,scan_counter);


     
     time = data1(:,1);
     trans_dualCam_raw = data1(:,2:end);
     trans_singleCam_raw = data2(:,2:end);
     trans_dualCamv2_raw = data3(:,2:end);
     trans_singleCamv2_raw = data4(:,2:end);

     %combine single and dual referencing for further noise-filtering
     diff_signal = trans_singleCamv2_raw - trans_dualCamv2_raw;
     diff_signal_mm = diff_signal;
     ind_mm = 400:800;
     diff_signal_mm(:,ind_mm) = movmean(diff_signal(:,ind_mm), 200, 2);
     trans_combinedCam_raw = trans_singleCamv2_raw - diff_signal_mm;

     if rem_bg > 0
         disp(['Use the first ' num2str(rem_bg) ' delays to estimate the background.']);
         bg_dualCam = mean(trans_dualCam_raw(1:rem_bg,:),1, 'omitnan');
         bg_singleCam = mean(trans_singleCam_raw(1:rem_bg,:),1, 'omitnan');
         bg_dualCamv2 = mean(trans_dualCamv2_raw(1:rem_bg,:),1, 'omitnan');
         bg_singleCamv2 = mean(trans_singleCamv2_raw(1:rem_bg,:),1, 'omitnan');
         bg_combinedCam = mean(trans_combinedCam_raw(1:rem_bg,:),1, 'omitnan');

         trans_dualCam_bgcor = trans_dualCam_raw - bg_dualCam + 1;
         trans_singleCam_bgcor = trans_singleCam_raw - bg_singleCam + 1;
         trans_dualCamv2_bgcor  = trans_dualCamv2_raw  - bg_dualCamv2 + 1;
         trans_singleCamv2_bgcor  = trans_singleCamv2_raw  - bg_singleCamv2 + 1;
         trans_combinedCam_bgcor  = trans_combinedCam_raw  - bg_combinedCam + 1;
     
     
     else
         disp(['No background substraction was performed.'])
         trans_dualCam_bgcor = trans_dualCam_raw;
         trans_singleCam_bgcor = trans_singleCam_raw;
         trans_dualCamv2_bgcor = trans_dualCamv2_raw;
         trans_singleCamv2_bgcor = trans_singleCamv2_raw;
         trans_combinedCam_bgcor = trans_combinedCam_raw;

     end

    

     pixrange = [300, 900]; 


     if scan_counter == 1
         trans_singleCam_bgcor_all = NaN*zeros(length(time), length(pixel), length(scan_id_vec));
         trans_dualCam_bgcor_all = NaN*zeros(length(time), length(pixel), length(scan_id_vec));
         trans_dualCamv2_bgcor_all = NaN*zeros(length(time), length(pixel), length(scan_id_vec));
         trans_singleCamv2_bgcor_all = NaN*zeros(length(time), length(pixel), length(scan_id_vec));
         trans_combinedCam_bgcor_all = NaN*zeros(length(time), length(pixel), length(scan_id_vec));
          
    end
     trans_singleCam_bgcor_all(:,:,scan_counter) = trans_singleCam_bgcor;
     trans_dualCam_bgcor_all(:,:,scan_counter) = trans_dualCam_bgcor;
     trans_dualCamv2_bgcor_all(:,:,scan_counter) = trans_dualCamv2_bgcor;
     trans_singleCamv2_bgcor_all(:,:,scan_counter) = trans_singleCamv2_bgcor;
     trans_combinedCam_bgcor_all(:,:,scan_counter) = trans_combinedCam_bgcor;
     
     ind = 1:scan_counter;
     plotdata_singleCam = mean(trans_singleCam_bgcor_all(:,:,ind),3, "omitnan");
     plotdata_dualCam = mean(trans_dualCam_bgcor_all(:,:,ind),3, "omitnan");
     plotdata_dualCamv2 = mean(trans_dualCamv2_bgcor_all(:,:,ind),3, "omitnan");
     plotdata_singleCamv2 = mean(trans_singleCamv2_bgcor_all(:,:,ind),3, "omitnan");
     plotdata_combinedCam = mean(trans_combinedCam_bgcor_all(:,:,ind),3, "omitnan");

     
     

     noise_ind = 500:550; 
     noise_level_single(scan_counter) = mean(std(plotdata_singleCam(:, noise_ind), 'omitnan'));
     noise_level_dual(scan_counter) = mean(std(plotdata_dualCam(:, noise_ind), 'omitnan'));
     noise_level_dualv2(scan_counter) = mean(std(plotdata_dualCamv2(:, noise_ind), 'omitnan'));
     noise_level_singlev2(scan_counter) = mean(std(plotdata_singleCamv2(:, noise_ind), 'omitnan'));
     noise_level_combined(scan_counter) = mean(std(plotdata_combinedCam(:, noise_ind), 'omitnan'));
 end
%% select data for further evaluation
TAS_data = plotdata_combinedCam';
time = time*1e12; %use ps units
time_cc = (time-time(1)-2.60); %in ps
dt = mean(diff(time_cc));


roi_data = TAS_data(300:850, :); %data outside the noise-levels (only used to get the colorbar ranges)
delta_c = 0.5*(max(max(roi_data))- min(min(roi_data)));
cmin = 1-delta_c;
cmax = 1+delta_c;



%% Chirp correction data
% data points for the fit (in format (pixel, time delay) )
%global fit_3rd; %used for plotting gvd-correction
a_fit_data = [16	4600;
199	4267;
280	4067;
373 3800;
424 3667;
549 3200;
626 2867;
750 2067;
775 1867;
843 1200];
a_fit_data (:,2) = a_fit_data(:,2)-(2000);
%imagesc(pixel,time, TAS_data); %plot data to estimate fitting points

a_fit_data = [18	3527; 199	3267; 303	3065; 403 2882;
504 2538; 626 2169; 705 1849; 806 1298];
a_fit_data (:,2) = a_fit_data(:,2)+(0*2000);
a_fit_data (:,2) = a_fit_data(:,2);

%Mai 2023
a_fit_data_wl3 = [425 22801; 444 23135; 468 23535; 500 23935; 529 24202; 558 24469; 586 24669; ...
    596 24736 ];

if chirp_correction_case == 1
    a_fit_data_wl = a_fit_data_wl3 ;
end


%% chirp correction fit
if correct_chirp_wavelengthbased == 0
    % fit the data FOR 2ORDER POLYNOMIAL

    x = a_fit_data(:,1);
    y = a_fit_data(:,2);
    fun = @(b,x) (b(1) + b(2)*(x)+b(3)*(x).^2);
    b0 = [[3163.19423526105,-0.00332167751805215,-9.68043779112535e-08]];
    b_0_linear_F= nlinfit(x,y,fun,b0);
    xfit = linspace(1,1024,1024);
    fit_2nd = fun(b_0_linear_F,xfit);
    % fit the data FOR 3ORDER POLYNOMIAL
    x = a_fit_data(:,1);
    y = a_fit_data(:,2);
    fun = @(b,x) (b(1) + b(2)*(x)+b(3)*(x).^2+ b(4)*(x).^3);
    b0 = [[3163.19423526105,-0.00332167751805215,-9.68043779112535e-08, 400]];
    b_0_linear_F= nlinfit(x,y,fun,b0);
    xfit = linspace(1,1024,1024);
    fit_3rd = fun(b_0_linear_F,xfit);
else
    % fit the data FOR 3ORDER POLYNOMIAL
    x = a_fit_data_wl(:,1);
    y = a_fit_data_wl(:,2);
    fun = @(b,x) (1e3*b(1) + 1e-3*b(2)*(x)+1e-6*b(3)*(x).^2+ 1e-9*b(4)*(x).^3);
    b0 = [-2,1e5,-1e5, 1e5];
    b_0_F= nlinfit(x,y,fun,b0);
    xfit = wave; 
    fit_3rd = fun(b_0_F,xfit);
    chirp_correction_time_offset = 1500;
    fit_3rd = fit_3rd-chirp_correction_time_offset; 
end

TAS_data_raw = TAS_data;




%% time chirp correction
if chirp_correction_case == 0
    %apply no chirp correction
else
    %tcor = round(fit_3rd/dt);
    tcor = round(fit_3rd*1e-3/dt);
    for i = 1:1024
        TAS_data(i,:) = circshift (TAS_data(i,:),-tcor(i)+round(time_cc/dt));
    end
end


%% draw 2D graph
% automatically approximate selected_time
%auto_select_time = 1;
if auto_select_time == 1
    ind = find(wave < 600 & wave > 450);
    signalStrenght_in_time = mean(abs(TAS_data(ind, :)-1),1);
    [peak, ind] = max(signalStrenght_in_time);
    if ind <= max(time_avg_width,5) || ind >= length(signalStrenght_in_time)-max(time_avg_width,5)
        disp('Warning: the auto select time might not work as expected.')
        ind = round(length(signalStrenght_in_time)/2); %put to the center
    end
    selected_time = time(ind);
else
    ind = 40;
    selected_time = time(ind);
end

TAS_data_2DM = medfilt2(TAS_data,[mfnum2 mfnum2]);

TAS_data_2DM= TAS_data_2DM-1;
time_cc = time-selected_time;


%% 1D selected data
if mfnum2 == 1
    TAS_data_time = TAS_data-1; %no filtering
else
    TAS_data_time = medfilt2(TAS_data,[mfnum2 mfnum2])-1; %2D median
end
dt = abs(time(2)-time(1));

disp(['Calculate the time average over ' num2str((time_avg_width*2+1)*dt) 'ps']);
ind0 = find(time == selected_time);
if mfnum == 1
    selected_time_data = mean(TAS_data_time(:,ind0-time_avg_width:ind0+time_avg_width),2); % no filtering
else
    selected_time_data = medfilt1(mean(TAS_data_time(:,ind0-time_avg_width:ind0+time_avg_width),2),mfnum); % involing 5 times near exact position
end


delta_absorbance = -log10(selected_time_data+1);



end