function [wave, delta_absorbance, time_cc, TAS_data_2DM] = get_deltaA_from_TASdata(measurementday, dataname, time_avg_width, rem_bg, chirp_correction_case )
%get absorbance change as function of wavelength from the TAS data.


%% physical constants
e = 1.60217662e-19;             %elementary charge (in C)
hbar = 6.62607015e-34/(2*pi);   %reduced Planck constant (in Js)
me = 9.109384e-31;              %electron rest mass (in kg)
c_light = 299792458;            %Speed of light in vacuum (in m/s)
kB = 1.380649e-23;              %Boltzmann constant (in J/K)
mu0 = 4*pi*1e-7;                %premeability of free space (in N/A^2)
e0 = 1/(mu0*c_light^2);         %permittivity of free space (in F/m)

%% sample information

data_path_whitelight = ['../RawData/' measurementday '/' measurementday '_Calibration/'];
if strcmp(measurementday, '20210907') 
    data_path_whitelight = ['../RawData/' '20210908' '/' '20210908' '_Calibration/'];
end
folder_path = ['../RawData/' measurementday '/'];

Samplename = erase(dataname,".txt"); %used as title and for storing 1D data
time_cc = 0.0; 
selected_time = 2.18+0.003823-0.06618;
auto_select_time = 1; %control variable
save_data = 0; %control variable

% control how to correct chirp
correct_chirp_wavelengthbased = 1;

remove_noisy_delay = 1; %control variable if noisy delay lines should be replaced for the evaluation




% default
mfnum2 = 1;
mfnum = 1;

mycolormap = diverging_map(linspace(0,1,100),[0,0,1],[1,0,0]);

cmin = 0.99995;cmax = 1.00005;
cmin2 = -0.0005;cmax2 = +0.0005;
Wavelength = 500; % selected wavelength (in nm) for signal change as function of time-delay

datafrom = [folder_path, dataname];
%% white light calibration
% read in data
%format: index, pixel, pumped, pixel, unpumped, pixel, transient
data400 = dlmread([data_path_whitelight, '400.csv'], ',', 1, 0);
data415 = dlmread([data_path_whitelight, '415.csv'], ',', 1, 0);
data488 = dlmread([data_path_whitelight, '488.csv'], ',', 1, 0);
try %sometimes the filter 536 is not available
    data531 = dlmread([data_path_whitelight, '531.csv'], ',', 1, 0);
    usediff_filter = 0;
catch
    data536 = dlmread([data_path_whitelight, '536.csv'], ',', 1, 0);
    usediff_filter = 1;
end
data542 = dlmread([data_path_whitelight, '542.csv'], ',', 1, 0);
data577 = dlmread([data_path_whitelight, '577.csv'], ',', 1, 0);

[pks, pxind400] = max(data400(:,5)); %find peak and corresponding index
px400 = data400(pxind400, 4); %get the corresponding pixel number
weightligthCalib = [px400, 400]; %wavelength in nm

filter415wavelength = flip([420.4, 615, 629.5, 650.9, 678.4, 712, 751.1, 797]);
[pks, pxind415] = findpeaks(data415(:,5)-mean(data415(950:end,5)), 'MinPeakHeight', 1000, 'MinPeakDistance', 20, 'MinPeakWidth', 5);
if length(pxind415) ~= length(filter415wavelength)
    if length(pxind415) == length(filter415wavelength)-1
        filter415wavelength = filter415wavelength(2:end); %sometimes the last peak is missing
    else
        disp('Error in calibartion with 415-filter!');
        return;
    end
else %for fitting it is better not to use the last peak value
    filter415wavelength = filter415wavelength(2:end);
    pxind415 = pxind415(2:end);
end
for counter = 1:length(filter415wavelength)
    weightligthCalib = [weightligthCalib; [data415(pxind415(counter), 4)], filter415wavelength(counter)];   
end

[pks, pxind488] = max(data488(:,5));
px488 = data488(pxind488, 4); %get the corresponding pixel number
weightligthCalib = [weightligthCalib; [px488, 488.5]]; 
if usediff_filter == 0
    [pks, pxind531] = max(data531(:,5));
    px531 = data531(pxind531, 4); %get the corresponding pixel number
    weightligthCalib = [weightligthCalib; [px531, 531]];   
else
    [pks, pxind536] = max(data536(:,5));
    px536 = data536(pxind536, 4); %get the corresponding pixel number
    weightligthCalib = [weightligthCalib; [px536, 536]];
end
[pks, pxind542] = max(data542(:,5));
px542 = data542(pxind542, 4); %get the corresponding pixel number
weightligthCalib = [weightligthCalib; [px542, 545.5]]; 
[pks, pxind577] = max(data577(:,5));
px577 = data577(pxind577, 4); %get the corresponding pixel number
weightligthCalib = [weightligthCalib; [px577, 577]]; 

%sort
[pixelnr, sorted_index] = sort(weightligthCalib(:,1));
weightligthCalib = [pixelnr, weightligthCalib(sorted_index,2)];

%linear fit
f_fit = fittype('m*x+q');
xfit = fit(weightligthCalib(:,1),weightligthCalib(:,2),f_fit,'StartPoint',[-1, 800]);
%quadratic fit
fit_ind = find(weightligthCalib(:,2) >= 480 & weightligthCalib(:,2) <= 660);
f_fit = fittype('a*x^2+m*x+q');
xfit = fit(weightligthCalib(fit_ind,1),weightligthCalib(fit_ind,2),f_fit,'StartPoint',[0, -1, 800]);

%store calibration parameters for wave = data(2:end,end)*calib_wave(2)+calib_wave(1);

calib_wave(1) = xfit.q;
calib_wave(2) = xfit.m;
calib_wave(3) = xfit.a;


%% load TAS data
data = load(datafrom);
time_orig = data(1,1:end-1)-data(1); %in fs

%correct time axis using the increments between scan positions
inc = round(mean(diff(time_orig*1e-15))/(2*0.1e-6)*c_light/5)*5;
dt = inc*2*0.1e-6/c_light;
time = (0:length(time_orig)-1)*dt*1e15; %in fs
disp(['Time axis corrected assuming ' num2str(inc) ' increments per delay step.']);

dt = time(2)-time(1);
data (end-1,end) = 1023;data (end,end) = 1024; 
pixel = data(2:end,end);
TAS_data = data(2:end,1:end-1);
TAS_data(TAS_data==0) = 1; % make 0 to 1 (blank part)

roi_data = TAS_data(300:850, :); %data outside the noise-levels (only used to get the colorbar ranges)
delta_c = 0.5*(max(max(roi_data))- min(min(roi_data)));
cmin = 1-delta_c;
cmax = 1+delta_c;
wave = data(2:end,end).^2*calib_wave(3)+data(2:end,end)*calib_wave(2)+calib_wave(1); %for quadratic fitting


%% Chirp correction data
global fit_3rd; %used for plotting gvd-correction

a_fit_data = [16	4600;
199	4267;
280	4067;
373 3800;
424 3667;
549 3200;
626 2867;
750 2067;
775 1867;
843 1200];
a_fit_data (:,2) = a_fit_data(:,2)-(2000);

a_fit_data = [18	3527; 199	3267; 303	3065; 403 2882;
504 2538; 626 2169; 705 1849; 806 1298];
a_fit_data (:,2) = a_fit_data(:,2)+(0*2000);
a_fit_data (:,2) = a_fit_data(:,2);



%March 2022
a_fit_data_wl = [393 131; 421 840; 456 1500; 491 2000; 506 2197; 552 2688; ...
    597 3050; 640 3250 ];

a_fit_data_wl2 = [393 131; 421 1200; 456 2200; 491 2700; 506 2850; 552 3450; ...
    597 3850; 640 4110 ];

if chirp_correction_case == 2
    a_fit_data_wl = a_fit_data_wl2 ;
end


%% chirp correction fit
if correct_chirp_wavelengthbased == 0
    % fit the data FOR 2ORDER POLYNOMIAL

    x = a_fit_data(:,1);
    y = a_fit_data(:,2);
    fun = @(b,x) (b(1) + b(2)*(x)+b(3)*(x).^2);
    b0 = [[3163.19423526105,-0.00332167751805215,-9.68043779112535e-08]];
    b_0_linear_F= nlinfit(x,y,fun,b0);
    xfit = linspace(1,1024,1024);
    fit_2nd = fun(b_0_linear_F,xfit);
    % fit the data FOR 3ORDER POLYNOMIAL
    x = a_fit_data(:,1);
    y = a_fit_data(:,2);
    fun = @(b,x) (b(1) + b(2)*(x)+b(3)*(x).^2+ b(4)*(x).^3);
    b0 = [[3163.19423526105,-0.00332167751805215,-9.68043779112535e-08, 400]];
    b_0_linear_F= nlinfit(x,y,fun,b0);
    xfit = linspace(1,1024,1024);
    fit_3rd = fun(b_0_linear_F,xfit);
else
    % fit the data FOR 3ORDER POLYNOMIAL
    x = a_fit_data_wl(:,1);
    y = a_fit_data_wl(:,2);
    fun = @(b,x) (1e3*b(1) + 1e-3*b(2)*(x)+1e-6*b(3)*(x).^2+ 1e-9*b(4)*(x).^3);
    b0 = [-2,1e5,-1e5, 1e5];
    b_0_F= nlinfit(x,y,fun,b0);
    xfit = wave; 
    fit_3rd = fun(b_0_F,xfit);
    chirp_correction_time_offset = 1500;
    fit_3rd = fit_3rd-chirp_correction_time_offset; 
end

TAS_data_raw = TAS_data;

%% detect and replace bad scan lines
if remove_noisy_delay == 1
    ind = find(wave < 600 & wave > 400);
    mean_sig = abs(mean(TAS_data_raw(ind,:),1)-1);
    [~, noisy_delay_ind] = findpeaks(mean_sig,'MinPeakHeight',1e-4, 'MinPeakDistance',2, 'MinPeakProminence',1e-4);
    disp([num2str(length(noisy_delay_ind)) ' noisy delay points were detected and are replaced by the mean value of the neighbors.'])
    for counter = 1:length(noisy_delay_ind)
        delay_ind = noisy_delay_ind(counter);
        TAS_data(:, delay_ind) = 0.5*(TAS_data(:, delay_ind-1) + TAS_data(:, delay_ind+1));
    end
end

%% TAS Background correction

if rem_bg > 0
    disp(['Use the first ' num2str(rem_bg) ' delays to estimate the background.']);
    BL_TAS_data = sum(TAS_data(:,1:rem_bg),2)/rem_bg;
    %plot(wave, BL_TAS_data)
    TAS_data = TAS_data - BL_TAS_data+1;
else
    disp(['No background substraction was performed.'])
    TAS_data = TAS_data;
end

%% time chirp correction
if chirp_correction_case == 0
    %apply no chirp correction
else
    if correct_chirp_wavelengthbased == 0
        % tcor = round(fit_2nd/dt);
        tcor = round(fit_3rd/dt);

        for i = 1:1024
            TAS_data(i,:) = circshift (TAS_data(i,:),-tcor(i)+round(time_cc/dt));
        end
    else
        tcor = round(fit_3rd/dt);

        for i = 1:1024
            TAS_data(i,:) = circshift (TAS_data(i,:),-tcor(i)+round(time_cc/dt));
        end
    end
end

%% rearange data array for saving
pixel = vertcat(0, pixel);
tT = vertcat (time, TAS_data);
data = horzcat (tT, pixel);
data (end,end) = 1024; %%correction of pixel number

dlamda = wave(2)-wave(1);

time = time*1e-3; %in ps
dt = time(2)-time(1);

TAS_data = data(2:end,1:end-1);
TAS_data(TAS_data==0) = 1; % make 0 to 1 (blank part)
%% draw 2D graph
% automatically approximate selected_time
if auto_select_time == 1
    ind = find(wave < 600 & wave > 450);
    signalStrenght_in_time = mean(abs(TAS_data(ind, :)-1),1);
    [peak, ind] = max(signalStrenght_in_time);
    if ind <= max(time_avg_width,5) || ind >= length(signalStrenght_in_time)-max(time_avg_width,5)
        disp('Warning: the auto select time might not work as expected.')
        ind = round(length(signalStrenght_in_time)/2); %put to the center
    end
    selected_time = time(ind);

end

TAS_data_2DM = medfilt2(TAS_data,[mfnum2 mfnum2]);
TAS_data_2DM(TAS_data_2DM==0) = 1; % make 0 to 1 (blank part)
TAS_data_2DM= TAS_data_2DM-1;
time_cc = time-selected_time;


%% 1D selected data
if mfnum2 == 1
    TAS_data_time = TAS_data-1; %no filtering
else
    TAS_data_time = medfilt2(TAS_data,[mfnum2 mfnum2])-1; %2D median
end
dt = abs(time(2)-time(1));

disp(['Calculate the time average over ' num2str((time_avg_width*2+1)*dt) 'ps']);
if mfnum == 1
    selected_time_data = mean(TAS_data_time(:,round(selected_time/dt+1)-time_avg_width:round(selected_time/dt+1)+time_avg_width),2); % no filtering
else
    selected_time_data = medfilt1(mean(TAS_data_time(:,round(selected_time/dt+1)-time_avg_width:round(selected_time/dt+1)+time_avg_width),2),mfnum); % involing 5 times near exact position
end

delta_absorbance = -log10(selected_time_data+1);



end