% clear
clear variables;
close all;
clc;
%% physical constants
e = 1.60217662e-19;             %elementary charge (in C)
hbar = 6.62607015e-34/(2*pi);   %reduced Planck constant (in Js)
me = 9.109384e-31;              %electron rest mass (in kg)
c_light = 299792458;            %Speed of light in vacuum (in m/s)
kB = 1.380649e-23;              %Boltzmann constant (in J/K)
mu0 = 4*pi*1e-7;                %premeability of free space (in N/A^2)
e0 = 1/(mu0*c_light^2);         %permittivity of free space (in F/m)
%% setup figure
fs = 12; %fontsize
linewidth = 2;
set(groot, 'defaultAxesTickLabelInterpreter','tex'); 
set(groot, 'defaultLegendInterpreter','tex');
set(groot, 'defaultTextInterpreter','tex');
set(groot, 'defaultAxesFontSize',fs);
set(groot, 'defaultTextFontSize',fs);
set(groot, 'defaultAxesLabelFontSize', fs);
set(groot, 'defaultAxesLabelFontSizeMultiplier', 1);
set(groot, 'defaultLineLineWidth', linewidth);
set(groot, 'defaultAxesLineWidth', linewidth);

fig1_name = 'SignalOverview_THzVsAC_v2';
fig1 = figure('Name', fig1_name, 'Position', [22 67 625 520]);
datapath_savefigure = '../Figures'; 
%% get AC data
AC_datapath = '../RawData/StarkFit_DataSets/';
sample1_filename = 'Sample1_AC/sam1_ac_test1';
sample2_filename = 'Sample2_AC/sam2_ac_test1';
for sample_id = 1:2
    if sample_id == 1
        alpha_pol = [90, 0]; %polarizations (in deg) (90deg is horizontal)
        alpha_inc = [45, 45]; %incidence angle (in deg)
        sample_filename = sample1_filename;
        titletext = 'Anthanthrene';
        legpos = 'northeast';
        xrange = [440, 550];
    elseif sample_id == 2
        alpha_pol = [90, 0]; %polarizations (in deg) (90deg is horizontal)
        alpha_inc = [37, 37]; %incidence angle (in deg)
        sample_filename = sample2_filename;
        titletext = 'TTF-BTD';
        legpos = 'northeast';
        xrange = [400, 650];
    end

    %get angle between probe and stark electric field
    for counter = 1:2
        if alpha_pol(counter) == 90
            n1 = 1.2; n2 = 1.5; %this are the same parameters as used in the Stark-fitting
            theta2 = asind(n1/n2*sind(alpha_inc(counter)));
            beta(counter) = round(90-theta2);
        elseif alpha_pol(counter) == 0
            beta(counter) = 90;
        else
            disp('Can not handle this polarization angle. Please check.')
        end
    end

    data_absfit = dlmread([AC_datapath, sample_filename, '_ABSFIT.txt'], '',1,0);
    data_stkfit1 = dlmread([AC_datapath, sample_filename, '_STKFIT1.txt'], '',1,0);
    data_stkfit2 = dlmread([AC_datapath, sample_filename, '_STKFIT2.txt'], '',1,0);

    if sample_id == 1
        data_absfit_sam1_ac = data_absfit;
        data_stkfit1_sam1_ac = data_stkfit1;
        data_stkfit2_sam1_ac = data_stkfit2;
        beta_sam1_ac = beta;
    else
        data_absfit_sam2_ac = data_absfit;
        data_stkfit1_sam2_ac = data_stkfit1;
        data_stkfit2_sam2_ac = data_stkfit2;
        beta_sam2_ac = beta;
    end
end

%% get THz data
THz_datapath ='../RawData/StarkFit_DataSets/';
sample1_filename = 'Sample1_Cuvette/sam1_cuvette_test11';
sample2_filename = 'Sample2_Cuvette/sam2_cuvette_test11';
for sample_id = 1:2
    if sample_id == 1
        alpha_pol = [90, 0]; %polarizations (in deg) (90deg is horizontal)
        alpha_inc = [90, 0]; %incidence angle (in deg)
        sample_filename = sample1_filename;
        titletext = 'Anthanthrene';
        legpos = 'northeast';
        xrange = [440, 550];
    elseif sample_id == 2
        alpha_pol = [90, 0]; %polarizations (in deg) (90deg is horizontal)
        alpha_inc = [90, 0]; %incidence angle (in deg)
        sample_filename = sample2_filename;
        titletext = 'TTF-BTD';
        legpos = 'northeast';
        xrange = [400, 650];
    end

    data_absfit = dlmread([THz_datapath, sample_filename, '_ABSFIT.txt'], '',1,0);
    data_stkfit1 = dlmread([THz_datapath, sample_filename, '_STKFIT1.txt'], '',1,0);
    data_stkfit2 = dlmread([THz_datapath, sample_filename, '_STKFIT2.txt'], '',1,0);

    %get angle between probe and stark electric field
    for counter = 1:2
        if alpha_pol(counter) == 90
            beta(counter) = 90-alpha_inc(counter);
        elseif alpha_pol(counter) == 0
            beta(counter) = 90;
        else
            disp('Can not handle this polarization angle. Please check.')
        end
    end

    if sample_id == 1
        data_absfit_sam1_thz = data_absfit;
        data_stkfit1_sam1_thz = data_stkfit1;
        data_stkfit2_sam1_thz = data_stkfit2;
        beta_sam1_thz = beta;
    else
        data_absfit_sam2_thz = data_absfit;
        data_stkfit1_sam2_thz = data_stkfit1;
        data_stkfit2_sam2_thz = data_stkfit2;
        beta_sam2_thz = beta;
    end
end
%% plot figure
figure(fig1); clf;

for sample_counter = 1:2

    col_ac1 = [0, 0.4, 0.7];
    col_ac2 = col_ac1 + [0.3, 0.3, 0.3]; %*0.5;
    col_thz1 = [0.70, 0.2, 0.0];
    col_thz2 = col_thz1 + [0.3, 0.3, 0.3];  % *0.5;

    switch sample_counter 
        case 2
            titletext = 'Anthanthrene';
            xax_lim = [400, 550];
            yax1_lim = [0, 1.1e5];
            yax2_lim = [-2000, 2500];
            data_absfit_thz = data_absfit_sam1_thz;
            data_absfit_ac = data_absfit_sam1_ac;
            data_stkfit1_ac = data_stkfit1_sam1_ac;
            data_stkfit2_ac = data_stkfit2_sam1_ac;
            data_stkfit1_thz = data_stkfit1_sam1_thz;
            data_stkfit2_thz = data_stkfit2_sam1_thz;
            beta_sam_ac = beta_sam1_ac;
            beta_sam_thz = beta_sam1_thz;
        case 1
            titletext = 'TTF-BTD';
            xax_lim = [400, 650];
            yax1_lim = [0, 5e4];
            yax2_lim = [-210, 250];
            data_absfit_thz = data_absfit_sam2_thz;
            data_absfit_ac = data_absfit_sam2_ac;
            data_stkfit1_ac = data_stkfit1_sam2_ac;
            data_stkfit2_ac = data_stkfit2_sam2_ac;
            data_stkfit1_thz = data_stkfit1_sam2_thz;
            data_stkfit2_thz = data_stkfit2_sam2_thz;
            beta_sam_ac = beta_sam2_ac;
            beta_sam_thz = beta_sam2_thz;
    end
    subplot(3,2,sample_counter); hold on;
    plot(1./data_absfit_ac(:,1)*1e9*1e-2, data_absfit_ac(:,2), '-', 'Color', col_ac1)
    plot(1./data_absfit_thz(:,1)*1e9*1e-2, data_absfit_thz(:,2), '-', 'Color', col_thz1)
    legend('77K (Conv.)', '295K (THz)', 'Location', ['northwest'])
    %xlabel('Wavelength (nm)')
    if sample_counter == 1
        ylabel({'Molar attenuation' 'coefficient \epsilon (M^{-1} cm^{-1})'});
    end
    grid on;
    title(titletext);
    xlim(xax_lim );
    ylim(yax1_lim);
    box on;

    subplot(3,2,2+sample_counter); hold on;
    plot(1./data_stkfit1_ac(:,1)*1e9*1e-2, data_stkfit1_ac(:,2), '-', 'Color', col_ac1)
    plot(1./data_stkfit2_ac(:,1)*1e9*1e-2, data_stkfit2_ac(:,2), '--',  'Color', col_ac2)

    legend({['Conv. (\chi = ' num2str(beta_sam_ac(1)) '°)'], ...
        ['Conv. (\chi = ' num2str(beta_sam_ac(2)) '°)']}, 'Location', 'northwest')
    ylim(yax2_lim);
    xlim(xax_lim);
    grid on; box on;

    subplot(3,2,4+sample_counter); hold on;
    plot(1./data_stkfit1_thz(:,1)*1e9*1e-2, data_stkfit1_thz(:,2), '-', 'Color', col_thz1);
    plot(1./data_stkfit2_thz(:,1)*1e9*1e-2, data_stkfit2_thz(:,2), '--', 'Color', col_thz2);

    legend({['THz (\chi = ' num2str(beta_sam_thz(1)) '°)'], ...
        ['THz (\chi = ' num2str(beta_sam_thz(2)) '°)']}, 'Location', 'northwest')


    xlabel('Wavelength (nm)')
    if sample_counter == 1
        hylabel = ylabel({'\Delta \epsilon (M^{-1} cm^{-1})'});
        set(hylabel, 'Position', [350.4785 314.4002 -1.0000]);
    end
    grid on; box on;
    xlim(xax_lim);
    ylim(yax2_lim);

end

fs = 12;
set(findall(gcf,'-property','FontSize'),'FontSize',fs)

fs2 = 16; 
hs = subplot(3,2,6);

h1 = text(330, 13700, 'a', 'Fontsize', fs2, 'FontWeight','bold');
h2 = text(520, 13700, 'b', 'Fontsize', fs2, 'FontWeight','bold');
h1 = text(330, 13700-6000, 'c', 'Fontsize', fs2, 'FontWeight','bold');
h2 = text(520, 13700-6000, 'd', 'Fontsize', fs2, 'FontWeight','bold');
h1 = text(330, 13700-2*6000, 'e', 'Fontsize', fs2, 'FontWeight','bold');
h2 = text(520, 13700-2*6000, 'f', 'Fontsize', fs2, 'FontWeight','bold');

subplot(3,2,1);
h1 = annotation('line', [1, 1]*0.355, [0.11, 0.92], 'LineStyle', ':', 'Color', [1,1,1]*0.5);
h2 = annotation('line', [1, 1]*0.285, [0.11, 0.25], 'LineStyle', ':', 'Color', [1,1,1]*0.5);
h2 = annotation('line', [1, 1]*0.285, [0.32, 0.55], 'LineStyle', ':', 'Color', [1,1,1]*0.5);
h2 = annotation('line', [1, 1]*0.285, [0.62, 0.85], 'LineStyle', ':', 'Color', [1,1,1]*0.5);

h3 = annotation('line', [1, 1]*0.814, [0.11, 0.92], 'LineStyle', ':', 'Color', [1,1,1]*0.5);
h4 = annotation('line', [1, 1]*0.795, [0.11, 0.92], 'LineStyle', ':', 'Color', [1,1,1]*0.5);


%% save figure
exportgraphics(fig1,[datapath_savefigure '/', fig1_name '.pdf'],'BackgroundColor','white','ContentType','image', 'Resolution', 300)
