% clear
clear variables;
close all;
clc;
%% physical constants
e = 1.60217662e-19;             %elementary charge (in C)
hbar = 6.62607015e-34/(2*pi);   %reduced Planck constant (in Js)
me = 9.109384e-31;              %electron rest mass (in kg)
c_light = 299792458;            %Speed of light in vacuum (in m/s)
kB = 1.380649e-23;              %Boltzmann constant (in J/K)
mu0 = 4*pi*1e-7;                %premeability of free space (in N/A^2)
e0 = 1/(mu0*c_light^2);         %permittivity of free space (in F/m)
%% setup figure
fs = 12; %fontsize
linewidth = 2;
set(groot, 'defaultAxesTickLabelInterpreter','tex'); 
set(groot, 'defaultLegendInterpreter','tex');
set(groot, 'defaultTextInterpreter','tex');
set(groot, 'defaultAxesFontSize',fs);
set(groot, 'defaultTextFontSize',fs);
set(groot, 'defaultAxesLabelFontSize', fs);
set(groot, 'defaultAxesLabelFontSizeMultiplier', 1);
set(groot, 'defaultLineLineWidth', linewidth);
set(groot, 'defaultAxesLineWidth', linewidth);

fig1_name = 'SignalOverview_TolVsEtOAc';
fig1 = figure('Name', fig1_name, 'Position', [22 67 625 520]);
datapath_savefigure = '../Figures'; 
%% get Tol and EtOAc data
THz_datapath = '../RawData/StarkFit_DataSets/';
sample1_filename = 'Sample1_Cuvette/sam1_cuvette_test13';
sample2_filename = 'AnthInEtOAc_THz2023/AnthInEtOAc_F3046';


for sample_id = 1:2
    if sample_id == 1
        alpha_pol = [90, 0]; %polarizations (in deg) (90deg is horizontal)
        alpha_inc = [90, 0]; %incidence angle (in deg)
        sample_filename = sample1_filename;
        titletext = 'Anthanthrene';
       
        legpos = 'northeast';
        xrange = [440, 550];
    elseif sample_id == 2
        alpha_pol = [90, 0]; %polarizations (in deg) (90deg is horizontal)
        alpha_inc = [90, 0]; %incidence angle (in deg)
        sample_filename = sample2_filename;
        titletext = 'Anthanthrene';
       
        legpos = 'northeast';
        xrange = [440, 550];
    end

    data_absfit = dlmread([THz_datapath, sample_filename, '_ABSFIT.txt'], '',1,0);
    data_stkfit1 = dlmread([THz_datapath, sample_filename, '_STKFIT1.txt'], '',1,0);
    data_stkfit2 = dlmread([THz_datapath, sample_filename, '_STKFIT2.txt'], '',1,0);

    %get angle between probe and stark electric field
    for counter = 1:2
        if alpha_pol(counter) == 90
            beta(counter) = 90-alpha_inc(counter);
        elseif alpha_pol(counter) == 0
            beta(counter) = 90;
        else
            disp('Can not handle this polarization angle. Please check.')
        end
    end

    if sample_id == 1
        data_absfit_sam1_thz = data_absfit;
        data_stkfit1_sam1_thz = data_stkfit1;
        data_stkfit2_sam1_thz = data_stkfit2;
        beta_sam1_thz = beta;
    else
        data_absfit_sam2_thz = data_absfit;
        data_stkfit1_sam2_thz = data_stkfit1;
        data_stkfit2_sam2_thz = data_stkfit2;
        beta_sam2_thz = beta;
    end
end

%% plot figure
figure(fig1); clf;

col_tol1 = [0, 0.4, 0.7];
col_tol2 = col_tol1 + [0.3, 0.3, 0.3]; %*0.5;
col_etoac1 = [0.70, 0.2, 0.0];
col_etoac2 = col_etoac1 + [0.3, 0.3, 0.3];  % *0.5;

titletext = 'Anthanthrene';
xax_lim = [400, 550];
yax1_lim = [0, 1.1e5];
yax2_lim = [-2000, 2500]*1.5;
data_absfit_tol = data_absfit_sam1_thz;
data_absfit_etoac = data_absfit_sam2_thz;
data_stkfit1_tol = data_stkfit1_sam1_thz;
data_stkfit2_tol = data_stkfit2_sam1_thz;
data_stkfit1_etoac = data_stkfit1_sam2_thz;
data_stkfit2_etoac = data_stkfit2_sam2_thz;
beta_sam_tol = beta_sam1_thz;
beta_sam_etoac = beta_sam2_thz;

subplot(3,1,1); hold on;
plot(1./data_absfit_tol(:,1)*1e9*1e-2, data_absfit_tol(:,2), '-', 'Color', col_tol1)
plot(1./data_absfit_etoac(:,1)*1e9*1e-2, data_absfit_etoac(:,2), '-', 'Color', col_etoac1)
legend('Toluene', 'EtOAc', 'Location', ['northwest'])

ylabel({'Molar attenuation' 'coefficient \epsilon (M^{-1} cm^{-1})'});

grid on;
title(titletext);
xlim(xax_lim );
ylim(yax1_lim);
box on;

subplot(3,1,2); hold on;
plot(1./data_stkfit1_tol(:,1)*1e9*1e-2, data_stkfit1_tol(:,2), '-', 'Color', col_tol1)
plot(1./data_stkfit2_tol(:,1)*1e9*1e-2, data_stkfit2_tol(:,2), '--',  'Color', col_tol2)

legend({['Toluene (\chi = ' num2str(beta_sam_tol(1)) '°)'], ...
    ['Toluene (\chi = ' num2str(beta_sam_tol(2)) '°)']}, 'Location', 'northwest')
ylim(yax2_lim);
xlim(xax_lim);
grid on; box on;

subplot(3,1,3); hold on;
plot(1./data_stkfit1_etoac(:,1)*1e9*1e-2, data_stkfit1_etoac(:,2), '-', 'Color', col_etoac1);
plot(1./data_stkfit2_etoac(:,1)*1e9*1e-2, data_stkfit2_etoac(:,2), '--', 'Color', col_etoac2);

legend({['EtOAc (\chi = ' num2str(beta_sam_etoac(1)) '°)'], ...
    ['EtOAc (\chi = ' num2str(beta_sam_etoac(2)) '°)']}, 'Location', 'northwest')


xlabel('Wavelength (nm)')
hylabel = ylabel({'\Delta \epsilon (M^{-1} cm^{-1})'});
set(hylabel, 'Position', [3.850309278350516e+02 5.377235361507961e+03 -1]);

grid on; box on;
xlim(xax_lim);
ylim(yax2_lim);


fs = 12;
set(findall(gcf,'-property','FontSize'),'FontSize',fs)

fs2 = 16; 
hs = subplot(3,1,3);

h1 = text(530, 21000, 'a', 'Fontsize', fs2, 'FontWeight','bold');
h2 = text(530, 21000-9200, 'b', 'Fontsize', fs2, 'FontWeight','bold');
h1 = text(530, 21000-2*9200, 'c', 'Fontsize', fs2, 'FontWeight','bold');

subplot(3,1,1);
h1 = annotation('line', [1, 1]*0.65, [0.11, 0.92], 'LineStyle', ':', 'Color', [1,1,1]*0.5);
h2 = annotation('line', [1, 1]*0.62, [0.11, 0.92], 'LineStyle', ':', 'Color', [1,1,1]*0.5);

%% save figure
exportgraphics(fig1,[datapath_savefigure '/', fig1_name '.pdf'],'BackgroundColor','white','ContentType','image', 'Resolution', 300)

